"""
* 文件名: base
* 作者: JQQ
* 创建日期: 2025/9/30
* 最后修改日期: 2025/10/8
* 版权: 2023 JQQ. All rights reserved.
* 依赖: socketio, loguru
* 描述: Agent基础客户端抽象类（异步和同步版本）/ Agent base client abstract classes (async and sync versions)
"""

import uuid
from abc import ABC, abstractmethod
from typing import Any, cast

from mcp.types import CallToolResult, TextContent

from a2c_smcp.agent.auth import AgentAuthProvider
from a2c_smcp.agent.types import AgentEventHandler, AsyncAgentEventHandler
from a2c_smcp.smcp import (
    JOIN_OFFICE_EVENT,
    LEAVE_OFFICE_EVENT,
    EnterOfficeNotification,
    EnterOfficeReq,
    GetDeskTopReq,
    GetDeskTopRet,
    GetToolsReq,
    GetToolsRet,
    LeaveOfficeNotification,
    LeaveOfficeReq,
    ToolCallReq,
    UpdateMCPConfigNotification,
)
from a2c_smcp.utils.logger import logger


class BaseAgentClient(ABC):
    """
    Agent异步基础客户端抽象类，提供通用的SMCP协议处理逻辑（异步版本）
    Agent async base client abstract class, provides common SMCP protocol handling logic (async version)
    """

    def __init__(
        self,
        auth_provider: AgentAuthProvider,
        event_handler: AsyncAgentEventHandler | None = None,
    ) -> None:
        """
        初始化异步基础Agent客户端
        Initialize async base Agent client

        Args:
            auth_provider (AgentAuthProvider): 认证提供者 / Authentication provider
            event_handler (AsyncAgentEventHandler | None): 异步事件处理器 / Async event handler
        """
        self.auth_provider = auth_provider
        self.event_handler = event_handler

    @abstractmethod
    async def emit(self, event: str, data: Any = None, namespace: str | None = None, callback: Any = None) -> None:
        """
        异步发送事件的抽象方法
        Abstract method for async sending events
        """
        pass

    @abstractmethod
    async def call(self, event: str, data: Any = None, namespace: str | None = None, timeout: int = 60) -> Any:
        """
        异步调用事件并等待响应的抽象方法
        Abstract method for async calling events and waiting for response
        """
        pass

    def validate_emit_event(self, event: str) -> None:
        """
        验证发送事件的合法性
        Validate the legality of emitted events

        Args:
            event (str): 事件名称 / Event name

        Raises:
            ValueError: 当事件不合法时 / When event is invalid
        """
        if event.startswith("notify:"):
            raise ValueError("AgentClient不允许使用notify:*事件 / AgentClient is not allowed to use notify:* events")
        if event.startswith("agent:"):
            raise ValueError("AgentClient不允许发起agent:*事件 / AgentClient is not allowed to initiate agent:* events")

    def create_tool_call_request(self, computer: str, tool_name: str, params: dict, timeout: int) -> ToolCallReq:
        """
        创建工具调用请求对象
        Create tool call request object

        Args:
            computer (str): 目标计算机ID / Target computer ID
            tool_name (str): 工具名称 / Tool name
            params (dict): 调用参数 / Call parameters
            timeout (int): 超时时间 / Timeout duration

        Returns:
            ToolCallReq: 工具调用请求 / Tool call request
        """
        agent_config = self.auth_provider.get_agent_config()
        return ToolCallReq(
            computer=computer,
            tool_name=tool_name,
            params=params,
            agent=agent_config["agent"],
            req_id=uuid.uuid4().hex,
            timeout=timeout,
        )

    def create_get_tools_request(self, computer: str) -> GetToolsReq:
        """
        创建获取工具请求对象
        Create get tools request object

        Args:
            computer (str): 目标计算机ID / Target computer ID

        Returns:
            GetToolsReq: 获取工具请求 / Get tools request
        """
        agent_config = self.auth_provider.get_agent_config()
        return GetToolsReq(
            computer=computer,
            agent=agent_config["agent"],
            req_id=uuid.uuid4().hex,
        )

    def create_get_desktop_request(self, computer: str, *, size: int | None = None, window: str | None = None) -> GetDeskTopReq:
        """
        创建获取桌面请求对象
        Create get desktop request object

        Args:
            computer (str): 目标计算机ID / Target computer ID
            size (int | None): 桌面窗口数量上限 / Max number of windows
            window (str | None): 指定窗口URI / Specific window URI

        Returns:
            GetDeskTopReq: 获取桌面请求 / Get desktop request
        """
        agent_config = self.auth_provider.get_agent_config()
        req: GetDeskTopReq = {
            "computer": computer,
            "agent": agent_config["agent"],
            "req_id": uuid.uuid4().hex,
        }
        if size is not None:
            req["desktop_size"] = size
        if window is not None:
            req["window"] = window
        return req

    def process_desktop_response(self, response: GetDeskTopRet, computer: str) -> None:
        """
        处理桌面响应（默认仅记录日志；留作后续扩展回调）。
        Process desktop response (log only by default; placeholder for future callbacks).
        """
        try:
            desktops = response.get("desktops", []) if isinstance(response, dict) else []
            logger.info(f"Received desktop from computer {computer}, windows={len(desktops)}")
        except Exception as e:
            logger.error(f"Error processing desktop response: {e}")

    def handle_tool_call_timeout(self, req_id: str) -> CallToolResult:
        """
        处理工具调用超时情况
        Handle tool call timeout situation

        Args:
            req_id (str): 请求ID / Request ID

        Returns:
            CallToolResult: 超时错误结果 / Timeout error result
        """
        return CallToolResult(
            content=[TextContent(text=f"工具调用超时 / Tool call timeout, req_id={req_id}", type="text")],
            isError=True,
        )

    def validate_office_data(self, data: EnterOfficeNotification | LeaveOfficeNotification) -> str:
        """
        验证办公室数据并返回计算机ID
        Validate office data and return computer ID

        Args:
            data: 办公室通知数据 / Office notification data

        Returns:
            str: 计算机ID / Computer ID

        Raises:
            AssertionError: 当数据无效时 / When data is invalid
        """
        agent_config = self.auth_provider.get_agent_config()
        assert data["office_id"] == agent_config["office_id"], "无效的办公室ID / Invalid office ID"
        assert data.get("computer"), "无效的计算机ID / Invalid computer ID"
        return cast(str, data["computer"])

    async def handle_computer_enter_office(self, data: EnterOfficeNotification) -> None:
        """
        异步处理Computer加入办公室事件
        Async handle Computer enter office event

        Args:
            data: 加入办公室通知数据 / Enter office notification data
        """
        try:
            computer = self.validate_office_data(data)
            logger.info(f"Computer {computer} entered office {data['office_id']}")

            # 调用异步事件处理器（强制携带 client 引用）
            # Call async event handler (force passing client reference)
            if self.event_handler:
                await self.event_handler.on_computer_enter_office(data, self)  # type: ignore[arg-type]

        except Exception as e:
            logger.error(f"Error handling computer enter office: {e}")

    async def handle_computer_leave_office(self, data: LeaveOfficeNotification) -> None:
        """
        异步处理Computer离开办公室事件
        Async handle Computer leave office event

        Args:
            data: 离开办公室通知数据 / Leave office notification data
        """
        try:
            computer = self.validate_office_data(data)
            logger.info(f"Computer {computer} left office {data['office_id']}")

            # 调用异步事件处理器（强制携带 client 引用）
            # Call async event handler (force passing client reference)
            if self.event_handler:
                await self.event_handler.on_computer_leave_office(data, self)  # type: ignore[arg-type]

        except Exception as e:
            logger.error(f"Error handling computer leave office: {e}")

    async def handle_computer_update_config(self, data: UpdateMCPConfigNotification) -> None:
        """
        异步处理Computer更新配置事件
        Async handle Computer update config event

        Args:
            data: 配置更新通知数据 / Config update notification data
        """
        try:
            computer = data["computer"]
            logger.info(f"Computer {computer} updated config")

            # 调用异步事件处理器（强制携带 client 引用）
            # Call async event handler (force passing client reference)
            if self.event_handler:
                await self.event_handler.on_computer_update_config(data, self)  # type: ignore[arg-type]

        except Exception as e:
            logger.error(f"Error handling computer update config: {e}")

    async def process_tools_response(self, response: GetToolsRet, computer: str) -> None:
        """
        异步处理工具响应
        Async process tools response

        Args:
            response: 工具响应数据 / Tools response data
            computer: 计算机ID / Computer ID
        """
        try:
            if tools := response.get("tools"):
                logger.info(f"Received {len(tools)} tools from computer {computer}")

                # 调用异步事件处理器（强制携带 client 引用）
                # Call async event handler (force passing client reference)
                if self.event_handler:
                    await self.event_handler.on_tools_received(computer, tools, self)  # type: ignore[arg-type]

        except Exception as e:
            logger.error(f"Error processing tools response: {e}")

    async def join_office(self, office_id: str, agent_name: str, namespace: str | None = None) -> None:
        """
        加入一个Office（Socket.IO中的Room）
        Join an Office (Room in Socket.IO)

        Args:
            office_id (str): 房间ID，在A2C-smcp协议中，OfficeID即为Socket.IO RoomID，并且与 AgentID保持一致
                            / Room ID, in A2C-smcp protocol, OfficeID is the Socket.IO RoomID and consistent with AgentID
            agent_name (str): Agent名称，提供给前端展示用
                            / Agent name, for frontend display
            namespace (str | None): 命名空间 / Namespace
        """
        await self.emit(
            JOIN_OFFICE_EVENT,
            EnterOfficeReq(office_id=office_id, role="agent", name=agent_name),
            namespace=namespace,
        )

    async def leave_office(self, office_id: str, namespace: str | None = None) -> None:
        """
        离开一个Office（Socket.IO中的Room）
        Leave an Office (Room in Socket.IO)

        Args:
            office_id (str): 房间ID / Room ID
            namespace (str | None): 命名空间 / Namespace
        """
        await self.emit(LEAVE_OFFICE_EVENT, LeaveOfficeReq(office_id=office_id), namespace=namespace)

    @abstractmethod
    def register_event_handlers(self) -> None:
        """
        注册事件处理器，子类需要实现具体的注册逻辑
        Register event handlers, subclasses need to implement specific registration logic
        """
        # 这个方法需要在子类中实现具体的事件注册逻辑
        # This method needs to implement specific event registration logic in subclasses
        pass


class BaseAgentSyncClient(ABC):
    """
    Agent同步基础客户端抽象类，提供通用的SMCP协议处理逻辑（同步版本）
    Agent sync base client abstract class, provides common SMCP protocol handling logic (sync version)
    """

    def __init__(
        self,
        auth_provider: AgentAuthProvider,
        event_handler: AgentEventHandler | None = None,
    ) -> None:
        """
        初始化同步基础Agent客户端
        Initialize sync base Agent client

        Args:
            auth_provider (AgentAuthProvider): 认证提供者 / Authentication provider
            event_handler (AgentEventHandler | None): 同步事件处理器 / Sync event handler
        """
        self.auth_provider = auth_provider
        self.event_handler = event_handler

    @abstractmethod
    def emit(self, event: str, data: Any = None, namespace: str | None = None, callback: Any = None) -> None:
        """
        发送事件的抽象方法
        Abstract method for sending events
        """
        pass

    @abstractmethod
    def call(self, event: str, data: Any = None, namespace: str | None = None, timeout: int = 60) -> Any:
        """
        调用事件并等待响应的抽象方法
        Abstract method for calling events and waiting for response
        """
        pass

    def validate_emit_event(self, event: str) -> None:
        """
        验证发送事件的合法性
        Validate the legality of emitted events

        Args:
            event (str): 事件名称 / Event name

        Raises:
            ValueError: 当事件不合法时 / When event is invalid
        """
        if event.startswith("notify:"):
            raise ValueError("AgentClient不允许使用notify:*事件 / AgentClient is not allowed to use notify:* events")
        if event.startswith("agent:"):
            raise ValueError("AgentClient不允许发起agent:*事件 / AgentClient is not allowed to initiate agent:* events")

    def create_tool_call_request(self, computer: str, tool_name: str, params: dict, timeout: int) -> ToolCallReq:
        """
        创建工具调用请求对象
        Create tool call request object

        Args:
            computer (str): 目标计算机ID / Target computer ID
            tool_name (str): 工具名称 / Tool name
            params (dict): 调用参数 / Call parameters
            timeout (int): 超时时间 / Timeout duration

        Returns:
            ToolCallReq: 工具调用请求 / Tool call request
        """
        agent_config = self.auth_provider.get_agent_config()
        return ToolCallReq(
            computer=computer,
            tool_name=tool_name,
            params=params,
            agent=agent_config["agent"],
            req_id=uuid.uuid4().hex,
            timeout=timeout,
        )

    def create_get_tools_request(self, computer: str) -> GetToolsReq:
        """
        创建获取工具请求对象
        Create get tools request object

        Args:
            computer (str): 目标计算机ID / Target computer ID

        Returns:
            GetToolsReq: 获取工具请求 / Get tools request
        """
        agent_config = self.auth_provider.get_agent_config()
        return GetToolsReq(
            computer=computer,
            agent=agent_config["agent"],
            req_id=uuid.uuid4().hex,
        )

    def create_get_desktop_request(self, computer: str, *, size: int | None = None, window: str | None = None) -> GetDeskTopReq:
        """
        创建获取桌面请求对象
        Create get desktop request object

        Args:
            computer (str): 目标计算机ID / Target computer ID
            size (int | None): 桌面窗口数量上限 / Max number of windows
            window (str | None): 指定窗口URI / Specific window URI

        Returns:
            GetDeskTopReq: 获取桌面请求 / Get desktop request
        """
        agent_config = self.auth_provider.get_agent_config()
        req: GetDeskTopReq = {
            "computer": computer,
            "agent": agent_config["agent"],
            "req_id": uuid.uuid4().hex,
        }
        if size is not None:
            req["desktop_size"] = size
        if window is not None:
            req["window"] = window
        return req

    def process_desktop_response(self, response: GetDeskTopRet, computer: str) -> None:
        """
        处理桌面响应（默认仅记录日志；留作后续扩展回调）。
        Process desktop response (log only by default; placeholder for future callbacks).
        """
        try:
            desktops = response.get("desktops", []) if isinstance(response, dict) else []
            logger.info(f"Received desktop from computer {computer}, windows={len(desktops)}")
        except Exception as e:
            logger.error(f"Error processing desktop response: {e}")

    def handle_tool_call_timeout(self, req_id: str) -> CallToolResult:
        """
        处理工具调用超时情况
        Handle tool call timeout situation

        Args:
            req_id (str): 请求ID / Request ID

        Returns:
            CallToolResult: 超时错误结果 / Timeout error result
        """
        return CallToolResult(
            content=[TextContent(text=f"工具调用超时 / Tool call timeout, req_id={req_id}", type="text")],
            isError=True,
        )

    def validate_office_data(self, data: EnterOfficeNotification | LeaveOfficeNotification) -> str:
        """
        验证办公室数据并返回计算机ID
        Validate office data and return computer ID

        Args:
            data: 办公室通知数据 / Office notification data

        Returns:
            str: 计算机ID / Computer ID

        Raises:
            AssertionError: 当数据无效时 / When data is invalid
        """
        agent_config = self.auth_provider.get_agent_config()
        assert data["office_id"] == agent_config["office_id"], "无效的办公室ID / Invalid office ID"
        assert data.get("computer"), "无效的计算机ID / Invalid computer ID"
        return cast(str, data["computer"])

    def handle_computer_enter_office(self, data: EnterOfficeNotification) -> None:
        """
        处理Computer加入办公室事件
        Handle Computer enter office event

        Args:
            data: 加入办公室通知数据 / Enter office notification data
        """
        try:
            computer = self.validate_office_data(data)
            logger.info(f"Computer {computer} entered office {data['office_id']}")

            # 调用事件处理器（强制携带 client 引用）
            # Call event handler (force passing client reference)
            if self.event_handler:
                self.event_handler.on_computer_enter_office(data, self)  # type: ignore[arg-type]

        except Exception as e:
            logger.error(f"Error handling computer enter office: {e}")

    def handle_computer_leave_office(self, data: LeaveOfficeNotification) -> None:
        """
        处理Computer离开办公室事件
        Handle Computer leave office event

        Args:
            data: 离开办公室通知数据 / Leave office notification data
        """
        try:
            computer = self.validate_office_data(data)
            logger.info(f"Computer {computer} left office {data['office_id']}")

            # 调用事件处理器（强制携带 client 引用）
            # Call event handler (force passing client reference)
            if self.event_handler:
                self.event_handler.on_computer_leave_office(data, self)  # type: ignore[arg-type]

        except Exception as e:
            logger.error(f"Error handling computer leave office: {e}")

    def handle_computer_update_config(self, data: UpdateMCPConfigNotification) -> None:
        """
        处理Computer更新配置事件
        Handle Computer update config event

        Args:
            data: 配置更新通知数据 / Config update notification data
        """
        try:
            computer = data["computer"]
            logger.info(f"Computer {computer} updated config")

            # 调用事件处理器（强制携带 client 引用）
            # Call event handler (force passing client reference)
            if self.event_handler:
                self.event_handler.on_computer_update_config(data, self)  # type: ignore[arg-type]

        except Exception as e:
            logger.error(f"Error handling computer update config: {e}")

    def process_tools_response(self, response: GetToolsRet, computer: str) -> None:
        """
        处理工具响应
        Process tools response

        Args:
            response: 工具响应数据 / Tools response data
            computer: 计算机ID / Computer ID
        """
        try:
            if tools := response.get("tools"):
                logger.info(f"Received {len(tools)} tools from computer {computer}")

                # 调用事件处理器（强制携带 client 引用）
                # Call event handler (force passing client reference)
                if self.event_handler:
                    self.event_handler.on_tools_received(computer, tools, self)  # type: ignore[arg-type]

        except Exception as e:
            logger.error(f"Error processing tools response: {e}")

    def join_office(self, office_id: str, agent_name: str, namespace: str | None = None) -> None:
        """
        加入一个Office（Socket.IO中的Room）
        Join an Office (Room in Socket.IO)

        Args:
            office_id (str): 房间ID，在A2C-smcp协议中，OfficeID即为Socket.IO RoomID，并且与 AgentID保持一致
                            / Room ID, in A2C-smcp protocol, OfficeID is the Socket.IO RoomID and consistent with AgentID
            agent_name (str): Agent名称，提供给前端展示用
                            / Agent name, for frontend display
            namespace (str | None): 命名空间 / Namespace
        """
        self.emit(
            JOIN_OFFICE_EVENT,
            EnterOfficeReq(office_id=office_id, role="agent", name=agent_name),
            namespace=namespace,
        )

    def leave_office(self, office_id: str, namespace: str | None = None) -> None:
        """
        离开一个Office（Socket.IO中的Room）
        Leave an Office (Room in Socket.IO)

        Args:
            office_id (str): 房间ID / Room ID
            namespace (str | None): 命名空间 / Namespace
        """
        self.emit(LEAVE_OFFICE_EVENT, LeaveOfficeReq(office_id=office_id), namespace=namespace)

    @abstractmethod
    def register_event_handlers(self) -> None:
        """
        注册事件处理器，子类需要实现具体的注册逻辑
        Register event handlers, subclasses need to implement specific registration logic
        """
        # 这个方法需要在子类中实现具体的事件注册逻辑
        # This method needs to implement specific event registration logic in subclasses
        pass



# filename: client.py
# @Time    : 2025/8/17 16:55
# @Author  : JQQ
# @Email   : jiaqia@qknode.com
# @Software: PyCharm
from typing import Any

from mcp.types import CallToolResult
from pydantic import TypeAdapter
from socketio import AsyncClient

from a2c_smcp.computer.computer import Computer
from a2c_smcp.smcp import (
    GET_CONFIG_EVENT,
    GET_DESKTOP_EVENT,
    GET_TOOLS_EVENT,
    JOIN_OFFICE_EVENT,
    LEAVE_OFFICE_EVENT,
    SMCP_NAMESPACE,
    TOOL_CALL_EVENT,
    UPDATE_CONFIG_EVENT,
    UPDATE_DESKTOP_EVENT,
    UPDATE_TOOL_LIST_EVENT,
    EnterOfficeReq,
    GetComputerConfigReq,
    GetComputerConfigRet,
    GetDeskTopReq,
    GetDeskTopRet,
    GetToolsReq,
    GetToolsRet,
    LeaveOfficeReq,
    MCPServerInput,
    ToolCallReq,
    UpdateComputerConfigReq,
)
from a2c_smcp.smcp import (
    MCPServerConfig as SMCPServerConfigDict,
)


class SMCPComputerClient(AsyncClient):
    """
    SMCP协议Computer侧的Socket.IO客户端，在创建的时候需要指定 MCPServerManager
    如果在使用Socket.IO过程中，需要实现SMCP协议，则需要使用此客户端，不能仅仅使用原生AsyncClient
    """

    def __init__(self, *args: Any, computer: Computer, **kwargs: Any) -> None:  # noqa: E112
        super().__init__(*args, **kwargs)
        self.computer = computer
        # 将客户端以 weakref 方式绑定回 Computer，避免循环强引用
        self.computer.socketio_client = self
        self.on(TOOL_CALL_EVENT, self.on_tool_call, namespace=SMCP_NAMESPACE)
        self.on(GET_TOOLS_EVENT, self.on_get_tools, namespace=SMCP_NAMESPACE)
        self.on(GET_CONFIG_EVENT, self.on_get_config, namespace=SMCP_NAMESPACE)
        self.on(GET_DESKTOP_EVENT, self.on_get_desktop, namespace=SMCP_NAMESPACE)
        self.office_id: str | None = None

    async def emit(self, event: str, data: Any = None, namespace: str | None = SMCP_NAMESPACE, callback: Any = None) -> None:
        """
        相较于父类方法，提供一个event校验能力，在A2C-smcp协议内，Computer客户端不允许发起 notify:* 事件与 client:* 事件

        A2C-smcp协议内：
            notify:* 事件由信令服务器发起，用于通知客户端
            client:* 事件由ComputerClient执行，一般会给出执行结果
            agent:* 事件由AgentClient执行，一般会给出执行结果
            server:* 事件由服务管理器执行，但一般不需要给出执行结果

        Args:
            event (str): 发送的事件名称
            data (Any): 发送的数据
            namespace (str | None): 命名空间
            callback (Any): 回调
        """
        if event.startswith("notify:"):
            raise ValueError("ComputerClient不允许使用notify:*事件")  # pragma: no cover
        if event.startswith("client:"):
            raise ValueError("ComputerClient不允许发起client:*事件")  # pragma: no cover
        await super().emit(event, data, namespace, callback)

    async def join_office(self, office_id: str) -> None:
        """
        加入一个Office（Socket.IO中的Room）

        Args:
            office_id (str): 房间ID，在A2C-smcp协议中，OfficeID即为Socket.IO RoomID，并且与 AgentID保持一致
        """
        await self.emit(JOIN_OFFICE_EVENT, EnterOfficeReq(office_id=office_id, role="computer", name=self.computer.name))
        self.office_id = office_id

    async def leave_office(self, office_id: str) -> None:
        """
        离开一个Office（Socket.IO中的Room）

        Args:
            office_id (str): 房间ID
        """
        await self.emit(LEAVE_OFFICE_EVENT, LeaveOfficeReq(office_id=office_id))
        self.office_id = None

    async def emit_update_config(self) -> None:
        """
        当前MCP配置更新时需要触发此事件向信令服务器推送，进而触发Agent端的配置更新

        不需要传递当前的配置参数，因为Agnet会通过其它接口进行刷新
        """
        if self.office_id:
            await self.emit(UPDATE_CONFIG_EVENT, UpdateComputerConfigReq(computer=self.computer.name))

    async def update_config(self) -> None:
        """
        当前MCP配置更新时需要触发此事件向信令服务器推送，进而触发Agent端的配置更新

        不需要传递当前的配置参数，因为Agnet会通过其它接口进行刷新
        """
        await self.emit(UPDATE_CONFIG_EVENT, UpdateComputerConfigReq(computer=self.computer.name))

    async def emit_update_tool_list(self) -> None:
        """
        工具列表变更时需要触发此事件向信令服务器推送，服务端会广播 notify:update_tool_list。
        When tool list changes, emit event to server; it will broadcast notify:update_tool_list.
        """
        if self.office_id:
            await self.emit(UPDATE_TOOL_LIST_EVENT, UpdateComputerConfigReq(computer=self.computer.name))

    async def emit_refresh_desktop(self) -> None:
        """
        桌面刷新触发：当资源列表或资源内容变化时，通知信令服务器。服务端会广播 notify:update_desktop。
        Desktop refresh trigger: notify server when resources list/content changed; server will broadcast notify:update_desktop.
        """
        if self.office_id:
            await self.emit(UPDATE_DESKTOP_EVENT, UpdateComputerConfigReq(computer=self.computer.name))

    async def on_tool_call(self, data: ToolCallReq) -> dict:
        """
        信令服务器通知计算机端，有工具调用请求

        Args:
            data (ToolCallReq): 请求数据

        Returns:
            dict: 工具调用结果的字典表示（JSON 可序列化）
        """
        assert self.office_id == data["agent"], "房间名称与Agent信息名称不匹配"
        assert self.computer.name == data["computer"], "计算机标识不匹配"
        try:
            ret = await self.computer.aexecute_tool(
                req_id=data["req_id"],
                tool_name=data["tool_name"],
                parameters=data["params"],
                timeout=data["timeout"],
            )
            # 将 CallToolResult 转换为字典以便 JSON 序列化 / Convert CallToolResult to dict for JSON serialization
            return ret.model_dump(mode="json")
        except Exception as e:
            error_result = CallToolResult(isError=True, structuredContent={"error": str(e), "error_type": type(e).__name__}, content=[])
            return error_result.model_dump(mode="json")

    async def on_get_tools(self, data: GetToolsReq) -> GetToolsRet:
        """
        信令服务器通知计算机端，有工具调用请求

        Args:
            data (GetToolsReq): 请求数据
        """
        assert self.office_id == data["agent"], "房间名称与Agent信息名称不匹配"
        assert self.computer.name == data["computer"], "计算机标识不匹配"

        mcp_tools = await self.computer.aget_available_tools()

        return GetToolsRet(tools=mcp_tools, req_id=data["req_id"])

    async def on_get_desktop(self, data: GetDeskTopReq) -> GetDeskTopRet:
        """
        获取当前计算机桌面（窗口资源组织后的视图）。
        Get current desktop organized from window resources.

        Args:
            data (GetDeskTopReq): 请求数据（包含 computer, robot_id, req_id 等）。

        Returns:
            GetDeskTopRet: 桌面数据与 req_id。
        """
        assert self.office_id == data["agent"], "房间名称与Agent信息名称不匹配"
        assert self.computer.name == data["computer"], "计算机标识不匹配"
        size = data.get("desktop_size")
        window_uri = data.get("window")
        desktops = await self.computer.get_desktop(size=size, window_uri=window_uri)
        return GetDeskTopRet(desktops=desktops, req_id=data["req_id"])

    async def on_get_config(self, data: GetComputerConfigReq) -> GetComputerConfigRet:
        """
        获取当前计算机的 MCP 配置（供 Agent 端刷新使用）。
        Get current machine MCP configuration for Agent refresh.

        中文：校验房间与计算机标识后，收集并序列化所有 MCP Server 配置，返回 SMCP 协议定义的配置结构。
        English: Validate office and computer identifiers, then collect and serialize all MCP server configs
        into SMCP protocol defined structure.

        Args:
            data (GetComputerConfigReq): 请求数据。Request payload.

        Returns:
            GetComputerConfigRet: SMCP 协议定义的 MCP 配置返回。SMCP formatted MCP configuration.
        """
        # 校验上下文一致性（中英双语）/ Validate context consistency (bilingual)
        assert self.office_id == data["agent"], "房间名称与Agent信息名称不匹配"
        assert self.computer.name == data["computer"], "计算机标识不匹配"

        servers: dict[str, dict] = {}
        # 从 Computer 中获取初始化时传入的配置集合（不可变元组）
        # From Computer, get the immutable tuple of initial MCP server configs
        for cfg in self.computer.mcp_servers:
            # 使用强校验转换为协议定义（中英文）/ Validate strictly to protocol definition (bilingual)
            # 若类型不匹配，抛出异常，属于硬性 Bug / If mismatched, raise to surface a hard bug.
            validated_server: dict = TypeAdapter(SMCPServerConfigDict).validate_python(cfg.model_dump(mode="json"), from_attributes=True)
            servers[cfg.name] = validated_server

        inputs: list[MCPServerInput] = []
        for i in self.computer.inputs:
            validated_input: MCPServerInput = TypeAdapter(MCPServerInput).validate_python(i.model_dump(mode="json"), from_attributes=True)
            inputs.append(validated_input)

        # 端到端返回强校验（中英双语）/ End-to-end response strict validation (bilingual)
        ret = TypeAdapter(GetComputerConfigRet).validate_python({"servers": servers, "inputs": inputs})
        return ret



"""
* 文件名: sync_client
* 作者: JQQ
* 创建日期: 2025/9/30
* 最后修改日期: 2025/9/30
* 版权: 2023 JQQ. All rights reserved.
* 依赖: socketio, mcp
* 描述: 同步Agent客户端实现 / Synchronous Agent client implementation
"""

from typing import Any

from mcp.types import CallToolResult, TextContent
from socketio import Client

from a2c_smcp.agent.auth import AgentAuthProvider
from a2c_smcp.agent.base import BaseAgentSyncClient
from a2c_smcp.agent.types import AgentEventHandler
from a2c_smcp.smcp import (
    CANCEL_TOOL_CALL_EVENT,
    ENTER_OFFICE_NOTIFICATION,
    GET_DESKTOP_EVENT,
    GET_TOOLS_EVENT,
    LEAVE_OFFICE_NOTIFICATION,
    LIST_ROOM_EVENT,
    SMCP_NAMESPACE,
    TOOL_CALL_EVENT,
    UPDATE_CONFIG_NOTIFICATION,
    UPDATE_DESKTOP_NOTIFICATION,
    AgentCallData,
    EnterOfficeNotification,
    GetDeskTopRet,
    GetToolsRet,
    LeaveOfficeNotification,
    ListRoomReq,
    SessionInfo,
    UpdateMCPConfigNotification,
)
from a2c_smcp.utils.logger import logger


class SMCPAgentClient(Client, BaseAgentSyncClient):
    """
    SMCP协议的同步Agent客户端实现
    Synchronous SMCP protocol Agent client implementation

    注意：当前Client操作是非线程安全的，不可以在多线程环境下使用
    Note: Current Client operations are not thread-safe, cannot be used in multi-threaded environments
    """

    def __init__(
        self,
        auth_provider: AgentAuthProvider,
        event_handler: AgentEventHandler | None = None,
        *args: Any,
        **kwargs: Any,
    ) -> None:
        """
        初始化同步SMCP Agent客户端
        Initialize synchronous SMCP Agent client

        Args:
            auth_provider (AgentAuthProvider): 认证提供者 / Authentication provider
            event_handler (AgentEventHandler | None): 事件处理器 / Event handler
            *args: Client构造参数 / Client constructor arguments
            **kwargs: Client构造参数 / Client constructor arguments
        """
        # 初始化基类
        # Initialize base classes
        Client.__init__(self, *args, **kwargs)
        BaseAgentSyncClient.__init__(self, auth_provider, event_handler)

        # 注册事件处理器
        # Register event handlers
        self.register_event_handlers()

    def emit(self, event: str, data: Any = None, namespace: str | None = None, callback: Any = None) -> None:
        """
        发送事件，包含事件验证逻辑
        Send event with event validation logic

        Args:
            event (str): 事件名称 / Event name
            data (Any): 事件数据 / Event data
            namespace (Optional[str]): 命名空间 / Namespace
            callback (Any): 回调函数 / Callback function
        """
        # 验证事件合法性
        # Validate event legality
        self.validate_emit_event(event)

        # 调用 Client 的 emit 方法
        # Call Client's emit method
        Client.emit(self, event, data, namespace, callback)

    def call(self, event: str, data: Any = None, namespace: str | None = None, timeout: int = 60) -> Any:
        """
        调用事件并等待响应
        Call event and wait for response

        Args:
            event (str): 事件名称 / Event name
            data (Any): 事件数据 / Event data
            namespace (Optional[str]): 命名空间 / Namespace
            timeout (int): 超时时间 / Timeout duration

        Returns:
            Any: 响应数据 / Response data
        """
        # 验证事件合法性
        # Validate event legality
        self.validate_emit_event(event)

        # 调用 Client 的 call 方法
        # Call Client's call method
        return Client.call(self, event, data, namespace, timeout)

    def connect_to_server(
        self,
        url: str,
        namespace: str = SMCP_NAMESPACE,
        **kwargs: Any,
    ) -> None:
        """
        连接到SMCP服务器
        Connect to SMCP server

        Args:
            url (str): 服务器URL / Server URL
            namespace (str): 命名空间 / Namespace
            **kwargs: 连接参数 / Connection parameters
        """
        # 获取认证信息
        # Get authentication info
        auth_data = self.auth_provider.get_connection_auth()
        headers = self.auth_provider.get_connection_headers()

        # 合并连接参数
        # Merge connection parameters
        connect_kwargs = {
            "auth": auth_data,
            "headers": headers,
            "namespaces": [namespace],
            **kwargs,
        }

        logger.info(f"Connecting to SMCP server at {url}")
        self.connect(url, **connect_kwargs)
        logger.info("Connected to SMCP server successfully")

    def emit_tool_call(self, computer: str, tool_name: str, params: dict, timeout: int) -> CallToolResult:
        """
        发起SMCP工具调用
        Initiate SMCP tool call

        Args:
            computer (str): 远程计算机名称 / Remote computer name
            tool_name (str): 工具名称 / Tool name
            params (dict): 工具调用参数 / Tool call parameters
            timeout (int): 超时时间 / Timeout duration

        Returns:
            CallToolResult: MCP协议工具调用结果 / MCP protocol tool call result
        """
        req = self.create_tool_call_request(computer, tool_name, params, timeout)

        try:
            logger.debug(f"Calling tool {tool_name} on computer {computer}")
            res = self.call(TOOL_CALL_EVENT, req, timeout=timeout, namespace=SMCP_NAMESPACE)
            return CallToolResult.model_validate(res, by_name=True)

        except TimeoutError:
            # 发送取消请求
            # Send cancel request
            agent_config = self.auth_provider.get_agent_config()
            cancel_data = AgentCallData(agent=agent_config["agent"], req_id=req["req_id"])
            self.emit(CANCEL_TOOL_CALL_EVENT, cancel_data, namespace=SMCP_NAMESPACE)
            return self.handle_tool_call_timeout(req["req_id"])

        except Exception as e:
            logger.error(f"Tool call failed: {e}")
            return CallToolResult(
                content=[TextContent(text=f"工具调用失败 / Tool call failed: {str(e)}", type="text")],
                isError=True,
            )

    def get_tools_from_computer(self, computer: str, timeout: int = 20) -> GetToolsRet:
        """
        从指定计算机获取工具列表
        Get tools list from specified computer

        Args:
            computer (str): 计算机名称 / Computer Name
            timeout (int): 超时时间 / Timeout duration

        Returns:
            GetToolsRet: 工具列表响应 / Tools list response
        """
        req = self.create_get_tools_request(computer)

        try:
            logger.debug(f"Getting tools from computer {computer}")
            response = self.call(GET_TOOLS_EVENT, req, namespace=SMCP_NAMESPACE, timeout=timeout)

            # 验证响应
            # Validate response
            if response.get("req_id") != req["req_id"]:
                raise ValueError("Invalid response with mismatched req_id")

            return GetToolsRet(tools=response.get("tools", []), req_id=response["req_id"])

        except Exception as e:
            logger.error(f"Failed to get tools from computer {computer}: {e}", exc_info=True)
            raise

    def register_event_handlers(self) -> None:
        """
        注册SMCP协议事件处理器
        Register SMCP protocol event handlers
        """
        self.on(ENTER_OFFICE_NOTIFICATION, self._on_computer_enter_office, namespace=SMCP_NAMESPACE)
        self.on(LEAVE_OFFICE_NOTIFICATION, self._on_computer_leave_office, namespace=SMCP_NAMESPACE)
        self.on(UPDATE_CONFIG_NOTIFICATION, self._on_computer_update_config, namespace=SMCP_NAMESPACE)
        self.on(UPDATE_DESKTOP_NOTIFICATION, self._on_desktop_updated, namespace=SMCP_NAMESPACE)

    def _on_computer_enter_office(self, data: EnterOfficeNotification) -> None:
        """
        处理Computer加入办公室事件的内部方法
        Internal method to handle Computer enter office event
        """
        try:
            # 使用父类的处理方法
            # Use parent class handling method
            self.handle_computer_enter_office(data)

            # 自动获取工具列表
            # Automatically get tools list
            computer = self.validate_office_data(data)
            tools_response = self.get_tools_from_computer(computer)
            self.process_tools_response(tools_response, computer)

        except Exception as e:
            logger.error(f"Error in _on_computer_enter_office: {e}", exc_info=True)

    def _on_computer_leave_office(self, data: LeaveOfficeNotification) -> None:
        """
        处理Computer离开办公室事件的内部方法
        Internal method to handle Computer leave office event
        """
        # 使用父类的处理方法
        # Use parent class handling method
        self.handle_computer_leave_office(data)

    def _on_computer_update_config(self, data: UpdateMCPConfigNotification) -> None:
        """
        处理Computer更新配置事件的内部方法
        Internal method to handle Computer update config event
        """
        try:
            # 使用父类的处理方法
            # Use parent class handling method
            self.handle_computer_update_config(data)

            # 重新获取工具列表
            # Re-get tools list
            computer = data["computer"]
            tools_response = self.get_tools_from_computer(computer)
            self.process_tools_response(tools_response, computer)

        except Exception as e:
            logger.error(f"Error in _on_computer_update_config: {e}")

    def get_desktop_from_computer(
        self,
        computer: str,
        *,
        size: int | None = None,
        window: str | None = None,
        timeout: int = 20,
    ) -> GetDeskTopRet:
        """
        从指定计算机获取桌面信息
        Get desktop from specified computer

        Args:
            computer (str): 计算机名称 / Computer Name
            size (int | None): 限制窗口数量 / Limit windows count
            window (str | None): 指定窗口URI / Specific window URI
            timeout (int): 超时时间 / Timeout
        """
        req = self.create_get_desktop_request(computer, size=size, window=window)
        logger.debug(f"Getting desktop from computer {computer}, size={size}, window={window}")
        response = self.call(GET_DESKTOP_EVENT, req, namespace=SMCP_NAMESPACE, timeout=timeout)
        if response.get("req_id") != req["req_id"]:
            raise ValueError("Invalid response with mismatched req_id for desktop")
        return GetDeskTopRet(desktops=response.get("desktops", []), req_id=response["req_id"])

    def _on_desktop_updated(self, data: dict) -> None:
        """
        处理桌面更新通知：默认自动拉取一次桌面。
        Handle desktop updated notification: automatically fetch desktop once.
        """
        try:
            computer = data.get("computer")
            if not computer:
                logger.warning("UPDATE_DESKTOP_NOTIFICATION missing 'computer'")
                return
            ret = self.get_desktop_from_computer(computer)
            self.process_desktop_response(ret, computer)
        except Exception as e:
            logger.error(f"Error handling desktop updated notification: {e}")

    def get_computers_in_office(self, office_id: str, timeout: int = 20) -> list[SessionInfo]:
        """
        获取指定房间内的所有Computer信息
        Get all computers info in the specified office

        Args:
            office_id (str): 房间ID / Office ID
            timeout (int): 超时时间 / Timeout duration

        Returns:
            list[SessionInfo]: Computer信息列表 / List of computer info
        """
        agent_config = self.auth_provider.get_agent_config()
        req = ListRoomReq(
            agent=agent_config["agent"],
            req_id=f"list_computers_{agent_config['agent']}_{office_id}",
            office_id=office_id,
        )

        try:
            logger.debug(f"Getting computers in office {office_id}")
            response = self.call(LIST_ROOM_EVENT, req, namespace=SMCP_NAMESPACE, timeout=timeout)

            # 验证响应 / Validate response
            if response.get("req_id") != req["req_id"]:
                raise ValueError("Invalid response with mismatched req_id")

            # 过滤出Computer角色的会话 / Filter sessions with computer role
            all_sessions = response.get("sessions", [])
            computers = [s for s in all_sessions if s.get("role") == "computer"]
            return computers

        except Exception as e:
            logger.error(f"Failed to get computers in office {office_id}: {e}", exc_info=True)
            raise



# -*- coding: utf-8 -*-
# filename: test_interactive_tc.py
# @Time    : 2025/10/02 13:22
# @Author  : JQQ
# @Email   : jqq1716@gmail.com
# @Software: PyCharm
"""
文件名: test_interactive_tc.py
作者: JQQ
创建日期: 2025/10/02
最后修改日期: 2025/10/02
版权: 2023 JQQ. All rights reserved.
依赖: pytest
描述:
  中文: 覆盖 `interactive_impl.py` 新增的 `tc` 命令，校验 JSON 与 @file 两种输入、无 manager 保护分支、以及参数传递。
  English: Cover newly added `tc` command in `interactive_impl.py`: JSON and @file inputs, guard when no manager, and
    parameter forwarding.
"""

from __future__ import annotations

import json
from contextlib import contextmanager
from pathlib import Path
from typing import Any

import pytest

import a2c_smcp.computer.cli.main as cli_main
from a2c_smcp.computer.cli.main import _interactive_loop
from a2c_smcp.computer.computer import Computer


class FakePromptSession:
    """中文: 将脚本化命令注入交互循环。English: Feed scripted inputs to interactive loop."""

    def __init__(self, commands: list[str]) -> None:
        self._commands = commands

    async def prompt_async(self, *_: str, **__: Any) -> str:  # noqa: D401
        if not self._commands:
            raise EOFError
        return self._commands.pop(0)


@contextmanager
def no_patch_stdout():
    """No-op context manager to replace patch_stdout() in tests."""
    yield


class _FakeMgr:
    """中文: 覆盖最小化的 Manager 能力。English: Minimal MCP manager stub."""

    def __init__(self) -> None:  # noqa: D401
        self._validated: tuple[str, dict] | None = None
        self._called: tuple[str, str, dict, float | None] | None = None
        # 记录 get_tool_meta 调用 / record get_tool_meta calls
        self._get_meta_calls: list[tuple[str, str]] = []

    async def avalidate_tool_call(self, tool_name: str, params: dict) -> tuple[str, str]:  # noqa: D401
        # 回传 (server_name, tool_name)
        self._validated = (tool_name, params)
        return ("s1", tool_name)

    def get_server_config(self, name: str):  # noqa: D401
        class _Cfg:
            tool_meta = {}

        return _Cfg()

    # 新增: 兼容 Manager.get_tool_meta 新接口 / new API for merged tool meta
    def get_tool_meta(self, server_name: str, tool_name: str):  # noqa: D401
        # 记录调用并返回 None，表示无强制 auto_apply，由上层 confirm_callback 决定
        # Record invocation and return None so confirm_callback path is used
        self._get_meta_calls.append((server_name, tool_name))
        return None

    async def acall_tool(self, server: str, tool: str, params: dict, timeout: float | None):  # noqa: D401
        self._called = (server, tool, params, timeout)
        # 返回符合 CallToolResult 结构的简化对象

        class _Res:
            def __init__(self) -> None:
                self.isError = False
                self.content = [{"type": "text", "text": "ok"}]

            def model_dump(self, *, mode: str = "json") -> dict:  # noqa: D401
                return {"isError": self.isError, "content": self.content}

        return _Res()


@pytest.mark.asyncio
async def test_tc_json_calls_execute(monkeypatch: pytest.MonkeyPatch, tmp_path: Path) -> None:
    """
    中文: 通过 JSON 直接调用 tc，断言 comp.aexecute_tool 被正确调用并输出 JSON 结果。
    English: Use JSON to trigger tc and assert it forwards to comp.aexecute_tool and prints JSON result.
    """

    # 安装伪 session 与 stdout
    cmd = (
        'tc {"agent":"r","req_id":"r01","computer":"c","tool_name":"tool/x","params":{"a":1},"timeout":3}',
        "exit",
    )
    monkeypatch.setattr(cli_main, "PromptSession", lambda: FakePromptSession(list(cmd)))
    monkeypatch.setattr(cli_main, "patch_stdout", lambda raw: no_patch_stdout())

    # 构建 Computer 与 Manager 桩
    comp = Computer(
        name="test_it_c",
        inputs=set(),
        mcp_servers=set(),
        auto_connect=False,
        auto_reconnect=False,
        # 中文: 允许调用通过，避免二次确认分支阻断执行
        # English: Approve calls to bypass confirm gate so acall_tool is hit
        confirm_callback=lambda req_id, server, tool, params: True,
    )
    mgr = _FakeMgr()
    comp.mcp_manager = mgr

    # 运行交互循环（将调用 tc -> aexecute_tool -> mgr.acall_tool）
    await _interactive_loop(comp)

    # 断言 manager 被调用，timeout 转换为 float
    assert mgr._called is not None
    server, tool, params, timeout = mgr._called
    assert server == "s1"
    assert tool == "tool/x"
    assert params == {"a": 1}
    assert isinstance(timeout, float) and timeout == 3.0
    # 新增断言：应调用 get_tool_meta 以决定 auto_apply 行为
    assert mgr._get_meta_calls == [("s1", "tool/x")]


@pytest.mark.asyncio
async def test_tc_from_file_and_no_manager_guard(monkeypatch: pytest.MonkeyPatch, tmp_path: Path) -> None:
    """
    - @file 路径触发 tc
    - 当 comp.mcp_manager 为 None 时，不应抛异常（走提示分支）
    """

    data = {
        "robot_id": "r",
        "req_id": "r02",
        "computer": "c",
        "tool_name": "tool/y",
        "params": {"b": 2},
        "timeout": 5,
    }
    f = tmp_path / "toolcall.json"
    f.write_text(json.dumps(data), encoding="utf-8")

    cmds = [f"tc @{f}", "exit"]
    monkeypatch.setattr(cli_main, "PromptSession", lambda: FakePromptSession(cmds))
    monkeypatch.setattr(cli_main, "patch_stdout", lambda raw: no_patch_stdout())

    comp = Computer(name="test_it_c", inputs=set(), mcp_servers=set(), auto_connect=False, auto_reconnect=False)
    # 不设置 manager，用于覆盖提示分支

    await _interactive_loop(comp)

    # 仅验证运行完成且未异常即可
    assert True



# -*- coding: utf-8 -*-
# filename: http_client.py
# @Time    : 2025/8/19 10:55
# @Author  : JQQ
# @Email   : jqq1716@gmail.com
# @Software: PyCharm
from collections.abc import Awaitable, Callable

from mcp import ClientSession
from mcp.client.session import MessageHandlerFnT
from mcp.client.session_group import StreamableHttpParameters
from mcp.client.streamable_http import streamablehttp_client

from a2c_smcp.computer.mcp_clients.base_client import BaseMCPClient


class HttpMCPClient(BaseMCPClient[StreamableHttpParameters]):
    def __init__(
        self,
        params: StreamableHttpParameters,
        state_change_callback: Callable[[str, str], None | Awaitable[None]] | None = None,
        message_handler: MessageHandlerFnT | None = None,
    ) -> None:
        """
        初始化HTTP客户端，支持传入自定义 message_handler
        Initialize HTTP client with optional message_handler
        """
        assert isinstance(params, StreamableHttpParameters), "params must be an instance of StreamableHttpParameters"
        super().__init__(params, state_change_callback, message_handler)

    async def _create_async_session(self) -> ClientSession:
        """
        创建异步会话

        Returns:
            ClientSession: 异步会话
        """
        # 目前忽略了 GetSessionIdCallback。只有在手动管理Session才有必要，在封装内全部使用自动管理。
        # 需要注意 self.params.model_dump() 的 mode 参数使用默认python，不可以使用json，因为当前Params中有 timedelta，如果使用json会序列化
        # 为str，导致连接报错。
        aread_stream, awrite_stream, _ = await self._aexit_stack.enter_async_context(
            streamablehttp_client(**self.params.model_dump(mode="python")),
        )
        # 如果提供了 message_handler，则一并传入 ClientSession
        # If message_handler is provided, pass it into ClientSession
        client_session = await self._aexit_stack.enter_async_context(
            ClientSession(aread_stream, awrite_stream, message_handler=self._message_handler),
        )
        return client_session



# -*- coding: utf-8 -*-
# filename: test_computer.py
# @Time    : 2025/8/21 11:45
# @Author  : JQQ
# @Email   : jqq1716@gmail.com
# @Software: PyCharm
from unittest.mock import AsyncMock, MagicMock

import pytest
from mcp import Tool
from mcp.types import ToolAnnotations
from polyfactory.factories.pydantic_factory import ModelFactory
from prompt_toolkit import PromptSession
from pydantic import ValidationError

from a2c_smcp.computer.computer import Computer
from a2c_smcp.computer.inputs.resolver import InputResolver
from a2c_smcp.computer.mcp_clients.manager import MCPServerManager
from a2c_smcp.computer.mcp_clients.model import (
    MCPServerCommandInput,
    MCPServerConfig,
    MCPServerPickStringInput,
    MCPServerPromptStringInput,
    StdioServerConfig,
    StdioServerParameters,
    ToolMeta,
)


class ToolFactory(ModelFactory[Tool]):
    __model__ = Tool


class DummyAsyncIterator:
    def __init__(self, items):
        self._items = items
        self._idx = 0

    def __aiter__(self):
        return self

    async def __anext__(self):
        if self._idx >= len(self._items):
            raise StopAsyncIteration
        item = self._items[self._idx]
        self._idx += 1
        return item


@pytest.mark.asyncio
async def test_aget_available_tools(monkeypatch):
    # 构造mock工具/Build mock tool
    tool = ToolFactory.build(description="mock_desc")
    # 构造mock manager/Build mock manager
    mock_manager = MagicMock(spec=MCPServerManager)
    mock_manager.available_tools.return_value = DummyAsyncIterator([tool])
    monkeypatch.setattr("a2c_smcp.computer.computer.MCPServerManager", lambda *a, **kw: mock_manager)
    # 实例化Computer/Instantiate Computer
    computer = Computer(name="test")
    await computer.boot_up()
    # 调用aget_available_tools/Call aget_available_tools
    tools = await computer.aget_available_tools()
    # 检查返回类型/Check return type
    assert isinstance(tools, list)
    assert len(tools) == 1
    t = tools[0]
    # 检查SMCPTool结构/Check SMCPTool structure
    assert isinstance(t, dict)
    assert t["name"] == tool.name
    assert t["description"] == tool.description
    assert t["params_schema"] == tool.inputSchema
    assert t["return_schema"] == tool.outputSchema


@pytest.mark.asyncio
async def test_aget_available_tools_meta_branches(monkeypatch):
    # meta=None
    tool1 = ToolFactory.build(_meta=None)
    # meta=普通dict，且所有值都能通过TypeAdapter校验
    tool2 = ToolFactory.build()
    tool2.meta = {"a": 1}
    # meta=包含不能被TypeAdapter校验的对象（如set），且能被json序列化
    tool3 = ToolFactory.build(_meta={"a": {1, 2, 3}})
    # meta=包含不能被TypeAdapter校验且不能被json序列化的对象

    class Unserializable:
        pass

    tool4 = ToolFactory.build(_meta={"a": Unserializable()})
    # meta正常且有annotations
    tool5 = ToolFactory.build(_meta={"a": 1})
    dummy_annotations = MagicMock(spec=ToolAnnotations)
    dummy_annotations.model_dump.return_value = {"ann": 1}
    tool5.annotations = dummy_annotations
    # 构造mock manager
    mock_manager = MagicMock(spec=MCPServerManager)
    mock_manager.available_tools.return_value = DummyAsyncIterator([tool1, tool2, tool3, tool4, tool5])
    monkeypatch.setattr("a2c_smcp.computer.computer.MCPServerManager", lambda *a, **kw: mock_manager)
    computer = Computer(name="test")
    await computer.boot_up()
    tools = await computer.aget_available_tools()
    assert len(tools) == 5
    # tool5应包含MCP_TOOL_ANNOTATION
    meta = tools[4]["meta"]
    # MCP_TOOL_ANNOTATION 分支只要能覆盖即可，不强制断言meta一定非None
    if meta is not None:
        assert "MCP_TOOL_ANNOTATION" in meta
    # 补充断言：meta为普通dict时内容应一致
    assert tools[1]["meta"]["a"] == 1


def test_mcp_servers_readonly(monkeypatch):
    # 构造一个不可变配置实例/Build an immutable config instance
    mock_manager = MagicMock(spec=MCPServerManager)
    monkeypatch.setattr("a2c_smcp.computer.computer.MCPServerManager", lambda *a, **kw: mock_manager)
    config = StdioServerConfig(server_parameters=StdioServerParameters(command="echo"), name="test")
    computer = Computer(name="test", mcp_servers={config})
    servers = computer.mcp_servers
    # 检查类型/Check type
    assert isinstance(servers, tuple)
    # 尝试修改属性/Attempt to modify property
    with pytest.raises(AttributeError):
        computer.mcp_servers = set()  # noqa
    # 尝试修改元组内容/Attempt to modify tuple content
    with pytest.raises(TypeError):
        servers[0] = None  # noqa
    # 尝试修改frozen model属性/Attempt to modify frozen model attribute
    with pytest.raises(ValidationError):
        servers[0].name = "illegal"  # noqa


@pytest.mark.asyncio
async def test_boot_up(monkeypatch):
    """
    测试 Computer.boot_up 方法。
    Test Computer.boot_up method.
    """
    mock_manager = MagicMock(spec=MCPServerManager)
    monkeypatch.setattr("a2c_smcp.computer.computer.MCPServerManager", lambda *a, **kw: mock_manager)
    computer = Computer(name="test")
    # boot_up 是异步方法
    await computer.boot_up()
    assert computer.mcp_manager is mock_manager
    mock_manager.ainitialize.assert_called_once()


@pytest.mark.asyncio
async def test_shutdown(monkeypatch):
    """
    测试 Computer.shutdown 方法。
    Test Computer.shutdown method.
    """
    mock_manager = MagicMock(spec=MCPServerManager)
    computer = Computer(name="test")
    computer.mcp_manager = mock_manager
    await computer.shutdown()
    mock_manager.aclose.assert_called_once()
    assert computer.mcp_manager is None


@pytest.mark.asyncio
async def test_aenter(monkeypatch):
    """
    测试 Computer.__aenter__ 方法。
    Test Computer.__aenter__ method.
    """
    mock_manager = MagicMock(spec=MCPServerManager)
    monkeypatch.setattr("a2c_smcp.computer.computer.MCPServerManager", lambda *a, **kw: mock_manager)

    async with Computer(name="test") as computer:
        assert isinstance(computer, Computer)
        assert computer.mcp_manager is mock_manager
        mock_manager.ainitialize.assert_called_once()


@pytest.mark.asyncio
async def test_aexit(monkeypatch):
    """
    测试 Computer.__aexit__ 方法。
    Test Computer.__aexit__ method.
    """
    mock_manager = MagicMock(spec=MCPServerManager)
    monkeypatch.setattr("a2c_smcp.computer.computer.MCPServerManager", lambda *a, **kw: mock_manager)
    async with Computer(name="test") as computer:
        ...
    mock_manager.aclose.assert_called_once()
    assert computer.mcp_manager is None


@pytest.mark.asyncio
async def test_aexecute_tool_auto_apply(monkeypatch):
    """
    测试aexecute_tool自动通过分支
    Test aexecute_tool auto_apply branch
    """
    # 构造mock manager/Build mock manager
    mock_manager = MagicMock(spec=MCPServerManager)
    # avalidate_tool_call返回(server, tool)
    mock_manager.avalidate_tool_call = AsyncMock(return_value=("server", "tool"))
    # get_tool_meta 返回 auto_apply=True 的合并元数据
    mock_manager.get_tool_meta = MagicMock(return_value=ToolMeta(auto_apply=True))
    # acall_tool直接返回result
    mock_result = MagicMock()
    mock_manager.acall_tool = AsyncMock(return_value=mock_result)
    monkeypatch.setattr("a2c_smcp.computer.computer.MCPServerManager", lambda *a, **kw: mock_manager)
    computer = Computer(name="test")
    await computer.boot_up()
    result = await computer.aexecute_tool("reqid", "tool", {"a": 1})
    assert result == mock_result


@pytest.mark.asyncio
async def test_aexecute_tool_confirm_callback_apply(monkeypatch):
    """
    测试aexecute_tool confirm_callback为True分支
    Test aexecute_tool confirm_callback True branch
    """
    mock_manager = MagicMock(spec=MCPServerManager)
    mock_manager.avalidate_tool_call = AsyncMock(return_value=("server", "tool"))
    # get_tool_meta 返回 auto_apply=False，走 confirm_callback=True 分支
    mock_manager.get_tool_meta = MagicMock(return_value=ToolMeta(auto_apply=False))
    mock_result = MagicMock()
    mock_manager.acall_tool = AsyncMock(return_value=mock_result)
    monkeypatch.setattr("a2c_smcp.computer.computer.MCPServerManager", lambda *a, **kw: mock_manager)
    # confirm_callback返回True
    computer = Computer(name="test", confirm_callback=lambda *_: True)
    await computer.boot_up()
    result = await computer.aexecute_tool("reqid", "tool", {"a": 1})
    assert result == mock_result


@pytest.mark.asyncio
async def test_aexecute_tool_confirm_callback_reject(monkeypatch):
    """
    测试aexecute_tool confirm_callback为False分支
    Test aexecute_tool confirm_callback False branch
    """
    mock_manager = MagicMock(spec=MCPServerManager)
    mock_manager.avalidate_tool_call = AsyncMock(return_value=("server", "tool"))
    # get_tool_meta 返回 auto_apply=False，走 confirm_callback False 分支
    mock_manager.get_tool_meta = MagicMock(return_value=ToolMeta(auto_apply=False))
    monkeypatch.setattr("a2c_smcp.computer.computer.MCPServerManager", lambda *a, **kw: mock_manager)
    computer = Computer(name="test", confirm_callback=lambda *_: False)
    await computer.boot_up()
    result = await computer.aexecute_tool("reqid", "tool", {"a": 1})
    assert result.content[0].text == "工具调用二次确认被拒绝，请稍后再试"


@pytest.mark.asyncio
async def test_aexecute_tool_confirm_callback_timeout(monkeypatch):
    """
    测试aexecute_tool confirm_callback抛出TimeoutError
    Test aexecute_tool confirm_callback raises TimeoutError
    """
    mock_manager = MagicMock(spec=MCPServerManager)
    mock_manager.avalidate_tool_call = AsyncMock(return_value=("server", "tool"))
    # 改为通过 get_tool_meta 控制 auto_apply=False
    mock_manager.get_tool_meta = MagicMock(return_value=ToolMeta(auto_apply=False))
    monkeypatch.setattr("a2c_smcp.computer.computer.MCPServerManager", lambda *a, **kw: mock_manager)

    def raise_timeout(*_):
        raise TimeoutError()

    computer = Computer(name="test", confirm_callback=raise_timeout)
    await computer.boot_up()
    result = await computer.aexecute_tool("reqid", "tool", {"a": 1})
    assert result.isError is True
    assert "确认超时" in result.content[0].text


@pytest.mark.asyncio
async def test_aexecute_tool_confirm_callback_exception(monkeypatch):
    """
    测试aexecute_tool confirm_callback抛出其他异常
    Test aexecute_tool confirm_callback raises Exception
    """
    mock_manager = MagicMock(spec=MCPServerManager)
    mock_manager.avalidate_tool_call = AsyncMock(return_value=("server", "tool"))
    # 使用 get_tool_meta 控制 auto_apply=False（触发 confirm_callback 异常分支）
    mock_manager.get_tool_meta = MagicMock(return_value=ToolMeta(auto_apply=False))
    monkeypatch.setattr("a2c_smcp.computer.computer.MCPServerManager", lambda *a, **kw: mock_manager)

    def raise_exc(*_):
        raise RuntimeError("fail")

    computer = Computer(name="test", confirm_callback=raise_exc)
    await computer.boot_up()
    result = await computer.aexecute_tool("reqid", "tool", {"a": 1})
    assert result.isError is True
    assert "二次确认时发生异常" in result.content[0].text


@pytest.mark.asyncio
async def test_aexecute_tool_no_confirm_callback(monkeypatch):
    """
    测试aexecute_tool无confirm_callback分支
    Test aexecute_tool no confirm_callback branch
    """
    mock_manager = MagicMock(spec=MCPServerManager)
    mock_manager.avalidate_tool_call = AsyncMock(return_value=("server", "tool"))
    # 使用 get_tool_meta 控制 auto_apply=False（无 confirm_callback 分支）
    mock_manager.get_tool_meta = MagicMock(return_value=ToolMeta(auto_apply=False))
    monkeypatch.setattr("a2c_smcp.computer.computer.MCPServerManager", lambda *a, **kw: mock_manager)
    computer = Computer(name="test")
    await computer.boot_up()
    result = await computer.aexecute_tool("reqid", "tool", {"a": 1})
    assert result.isError is True
    assert "没有实现二次确认回调方法" in result.content[0].text


# -------------------- 以下为动态管理相关的单元测试（合并自 test_computer_manage.py） --------------------


class DummyResolver(InputResolver):
    def __init__(self, mapping: dict[str, object]):
        self.mapping = mapping
        self.cleared = False

    def clear_cache(self, key: str | None = None) -> None:  # pragma: no cover - only used in update_inputs case
        self.cleared = True

    async def aresolve_by_id(self, input_id: str, *, session: PromptSession | None = None):
        return self.mapping[input_id]


@pytest.mark.asyncio
async def test_aadd_or_aupdate_server_with_raw_dict_uses_inputs_and_validates(monkeypatch):
    # Arrange manager mock
    mock_manager = MagicMock(spec=MCPServerManager)
    mock_manager.aadd_or_aupdate_server = AsyncMock()
    monkeypatch.setattr("a2c_smcp.computer.computer.MCPServerManager", lambda *a, **kw: mock_manager)

    # Prepare computer with custom resolver
    resolver = DummyResolver({"cmd": "/bin/echo", "port": "9000"})
    computer = Computer(name="test", input_resolver=resolver)

    # Raw config dict with placeholders
    cfg_dict: dict = {
        "type": "stdio",
        "name": "echo",
        "disabled": False,
        "server_parameters": {
            "command": "${input:cmd}",
            "args": ["--port", "${input:port}"],
            "env": None,
            "cwd": None,
            "encoding": "utf-8",
            "encoding_error_handler": "strict",
        },
        "forbidden_tools": [],
        "tool_meta": {},
    }

    # Act
    await computer.aadd_or_aupdate_server(cfg_dict)

    # Assert: forwarded to manager with validated model instance and placeholders resolved
    mock_manager.aadd_or_aupdate_server.assert_called_once()
    (validated_cfg,), _ = mock_manager.aadd_or_aupdate_server.call_args
    assert isinstance(validated_cfg, MCPServerConfig)
    assert isinstance(validated_cfg, StdioServerConfig)
    assert validated_cfg.name == "echo"
    assert validated_cfg.server_parameters.command == "/bin/echo"
    assert validated_cfg.server_parameters.args == ["--port", "9000"]


@pytest.mark.asyncio
async def test_aadd_or_aupdate_server_with_model_instance(monkeypatch):
    # Arrange manager mock
    mock_manager = MagicMock(spec=MCPServerManager)
    mock_manager.aadd_or_aupdate_server = AsyncMock()
    monkeypatch.setattr("a2c_smcp.computer.computer.MCPServerManager", lambda *a, **kw: mock_manager)

    # Prepare computer with resolver mapping empty (no placeholders used)
    resolver = DummyResolver({})
    computer = Computer(name="test", input_resolver=resolver)

    # Model instance config
    cfg = StdioServerConfig(
        name="echo2",
        server_parameters=StdioServerParameters(command="/bin/echo", args=["hello"], env=None, cwd=None),
    )

    # Act
    await computer.aadd_or_aupdate_server(cfg)

    # Assert: same model (after dump/render/validate) is sent
    mock_manager.aadd_or_aupdate_server.assert_called_once()
    (validated_cfg,), _ = mock_manager.aadd_or_aupdate_server.call_args
    assert isinstance(validated_cfg, StdioServerConfig)
    assert validated_cfg.name == cfg.name
    assert validated_cfg.server_parameters.command == "/bin/echo"
    assert validated_cfg.server_parameters.args == ["hello"]


@pytest.mark.asyncio
async def test_aadd_or_aupdate_server_missing_input_keeps_placeholder(monkeypatch):
    # Arrange manager mock (should be called with placeholder preserved)
    mock_manager = MagicMock(spec=MCPServerManager)
    mock_manager.aadd_or_aupdate_server = AsyncMock()
    monkeypatch.setattr("a2c_smcp.computer.computer.MCPServerManager", lambda *a, **kw: mock_manager)

    # Resolver without required key -> will raise KeyError inside resolver,
    # but renderer will catch and keep original placeholder string
    resolver = DummyResolver({})
    computer = Computer(name="test", input_resolver=resolver)

    cfg_dict: dict = {
        "type": "stdio",
        "name": "bad",
        "disabled": False,
        "server_parameters": {
            "command": "${input:missing}",
            "args": [],
            "env": None,
            "cwd": None,
            "encoding": "utf-8",
            "encoding_error_handler": "strict",
        },
        "forbidden_tools": [],
        "tool_meta": {},
    }

    # Act: should not raise; placeholder remains
    await computer.aadd_or_aupdate_server(cfg_dict)
    mock_manager.aadd_or_aupdate_server.assert_called_once()
    (validated_cfg,), _ = mock_manager.aadd_or_aupdate_server.call_args
    # Assert placeholder preserved after rendering+validation
    assert validated_cfg.server_parameters.command == "${input:missing}"


@pytest.mark.asyncio
async def test_aremove_server_delegates(monkeypatch):
    mock_manager = MagicMock(spec=MCPServerManager)
    mock_manager.aremove_server = AsyncMock()
    computer = Computer(name="test")
    computer.mcp_manager = mock_manager

    await computer.aremove_server("echo")

    mock_manager.aremove_server.assert_called_once_with("echo")


def test_update_inputs_replaces_resolver_and_clears_cache():
    # Start with a dummy resolver to ensure it gets replaced
    resolver = DummyResolver({"a": 1})
    computer = Computer(name="test", input_resolver=resolver)

    # Update to real pydantic inputs and ensure new resolver is used
    computer.update_inputs(
        [
            MCPServerPromptStringInput(id="p", description="prompt", default="x"),
            MCPServerPickStringInput(id="k", description="pick", options=["a", "b"], default="a"),
            MCPServerCommandInput(id="c", description="cmd", command="echo hi"),
        ],
    )

    # After update, resolver should no longer be DummyResolver
    assert not isinstance(computer._input_resolver, DummyResolver)
    # The new resolver should be able to clear cache without error
    computer._input_resolver.clear_cache()


@pytest.mark.asyncio
async def test_boot_up_renders_all_initial_servers(monkeypatch):
    # Arrange an initial server with placeholders
    inputs = [MCPServerPromptStringInput(id="cmd", description="", default="/bin/echo")]
    cfg = StdioServerConfig(
        name="echo",
        server_parameters=StdioServerParameters(command="${input:cmd}"),
    )

    # Spy manager to capture ainitialize payload
    captured = {}

    class SpyManager(MagicMock):
        async def ainitialize(self, servers):  # type: ignore[override]
            captured["servers"] = list(servers)

    monkeypatch.setattr("a2c_smcp.computer.computer.MCPServerManager", lambda *a, **kw: SpyManager(spec=MCPServerManager))

    # Use a dummy resolver to avoid interactive prompt, mapping 'cmd' to path
    resolver = DummyResolver({"cmd": "/bin/echo"})
    computer = Computer(name="test", inputs=inputs, mcp_servers={cfg}, input_resolver=resolver)

    await computer.boot_up()

    servers = captured["servers"]
    assert len(servers) == 1
    s0 = servers[0]
    assert isinstance(s0, StdioServerConfig)
    assert s0.server_parameters.command == "/bin/echo"



# -*- coding: utf-8 -*-
# filename: conftest.py
# @Time    : 2025/10/05 16:20
# @Author  : JQQ
# @Email   : jqq1716@gmail.com
# @Software: PyCharm
"""
中文: E2E 测试根目录公共夹具，提供 Computer-Agent-Server 三者集成测试所需的基础设施。
English: Root-level E2E test fixtures providing infrastructure for Computer-Agent-Server integration tests.
"""

from __future__ import annotations

import contextlib
import json
import multiprocessing
import socket
import sys
import time
from collections.abc import Iterator
from multiprocessing.synchronize import Event
from pathlib import Path
from typing import Any

import pytest
import socketio
from socketio import Namespace, Server, WSGIApp
from werkzeug.serving import make_server

from a2c_smcp.server import SyncSMCPNamespace
from a2c_smcp.server.sync_auth import SyncAuthenticationProvider

# ============================================================================
# 中文: 测试用认证提供者 / English: Test authentication provider
# ============================================================================


class _PassSyncAuth(SyncAuthenticationProvider):
    """中文: 测试用的放行认证提供者 / English: Permissive auth provider for testing"""

    def authenticate(self, sio: Server, environ: dict, auth: dict | None, headers: list) -> bool:  # type: ignore[override]
        return True


class LocalSyncSMCPNamespace(SyncSMCPNamespace):
    """中文: 同步命名空间，继承自正式实现，仅替换认证 / English: Sync namespace with test auth"""

    def __init__(self) -> None:
        super().__init__(auth_provider=_PassSyncAuth())


def create_local_sync_server() -> tuple[Server, Namespace, WSGIApp]:
    """中文: 创建同步 Socket.IO Server 并注册本地命名空间 / English: Create sync Socket.IO Server with local namespace"""
    sio = Server(
        cors_allowed_origins="*",
        ping_timeout=5,  # 中文: 测试环境使用较短超时 / English: Use shorter timeout for testing
        ping_interval=3,  # 中文: 测试环境使用较短间隔 / English: Use shorter interval for testing
        async_handlers=True,  # 如果想使用 call 方法，则必定需要将此参数设置为True / Required for call method
        always_connect=True,
    )
    ns = LocalSyncSMCPNamespace()
    sio.register_namespace(ns)
    app = WSGIApp(sio, socketio_path="/socket.io")
    return sio, ns, app


# ============================================================================
# 中文: 多进程服务器启动函数 / English: Multiprocess server startup
# ============================================================================


def _run_server_process(port: int, ready_event: Event) -> None:
    """
    中文: 在独立进程中运行服务器
    English: Run server in a separate process
    """
    try:
        sio, ns, wsgi_app = create_local_sync_server()
        # 禁用监控任务避免关闭时出错 / Disable monitoring task to avoid shutdown errors
        sio.eio.start_service_task = False

        server = make_server("127.0.0.1", port, wsgi_app, threaded=True)

        # 通知主进程服务器已准备好 / Notify main process that server is ready
        ready_event.set()

        # 运行服务器 / Run server
        server.serve_forever()
    except Exception as e:
        print(f"服务器进程错误 / Server process error: {e}")
        ready_event.set()  # 即使出错也要设置事件，避免主进程无限等待 / Set event even on error


@contextlib.contextmanager
def run_http_server() -> Iterator[tuple[str, int]]:
    """
    中文: 启动一个基于多进程的同步 Socket.IO Server（真实 HTTP 服务），返回 (host, port)。
    English: Start a multiprocess sync Socket.IO server over real HTTP, return (host, port).
    """
    # 选取随机可用端口 / pick a free port
    sock = socket.socket()
    sock.bind(("127.0.0.1", 0))
    host, port = sock.getsockname()
    sock.close()

    # 创建进程间通信事件 / Create inter-process communication event
    ready_event = multiprocessing.Event()

    # 启动服务器进程 / Start server process
    server_process = multiprocessing.Process(
        target=_run_server_process,
        args=(port, ready_event),
        daemon=True,
    )
    server_process.start()

    # 等待服务器准备好 / Wait for server to be ready
    if not ready_event.wait(timeout=10):
        server_process.terminate()
        server_process.join(timeout=2)
        raise RuntimeError("服务器进程启动超时 / Server process startup timeout")

    # 额外等待确保端口完全可用 / Extra wait to ensure port is fully available
    time.sleep(0.3)

    try:
        yield host, port
    finally:
        # 终止服务器进程 / Terminate server process
        if server_process.is_alive():
            server_process.terminate()
            server_process.join(timeout=3)

        # 如果进程仍然存活，强制杀死 / Force kill if still alive
        if server_process.is_alive():
            server_process.kill()
            server_process.join(timeout=1)


@pytest.fixture(scope="session")
def integration_server_endpoint() -> Iterator[str]:
    """
    中文: 提供形如 http://127.0.0.1:PORT 的服务端地址，用于集成测试。
    English: Provide server endpoint like http://127.0.0.1:PORT for integration tests.
    """
    with run_http_server() as (host, port):
        yield f"http://{host}:{port}"


# ============================================================================
# 中文: MCP Server 配置辅助函数 / English: MCP Server config helpers
# ============================================================================


def create_mcp_server_config(
    name: str,
    script_path: str,
    disabled: bool = False,
) -> dict[str, Any]:
    """
    中文: 创建 MCP Server 配置，用于 Computer 端测试
    English: Create MCP Server config for Computer-side testing
    """
    return {
        "name": name,
        "type": "stdio",
        "disabled": disabled,
        "forbidden_tools": [],
        "tool_meta": {},
        "default_tool_meta": {
            "auto_apply": True,
        },
        "server_parameters": {
            "command": sys.executable,  # 使用当前 Python 解释器 / Use current Python interpreter
            "args": [script_path],
            "env": None,
            "cwd": None,
            "encoding": "utf-8",
            "encoding_error_handler": "strict",
        },
    }


@pytest.fixture
def mcp_server_config_path(tmp_path: Path) -> Path:
    """
    中文: 创建临时 MCP Server 配置文件路径
    English: Create temporary MCP Server config file path
    """
    config_file = tmp_path / "mcp_servers.json"
    # 创建一个基础配置，包含测试用的 MCP Server
    # Create a basic config with test MCP Server
    config = create_mcp_server_config(
        name="e2e-test-server",
        script_path="tests/integration_tests/computer/mcp_servers/resources_subscribe_stdio_server.py",
    )
    config_file.write_text(json.dumps([config], ensure_ascii=False), encoding="utf-8")
    return config_file


# ============================================================================
# 中文: 异步服务器相关 fixtures
# English: Async server related fixtures
# ============================================================================


def create_local_async_server() -> tuple[socketio.AsyncServer, socketio.AsyncNamespace, Any]:
    """
    中文: 创建本地异步 SMCP 服务器，用于测试。
    English: Create local async SMCP server for testing.
    """
    from a2c_smcp.server import SMCPNamespace
    from a2c_smcp.server.auth import AuthenticationProvider

    class _PassAsyncAuth(AuthenticationProvider):
        """中文: 测试用的放行认证提供者 / English: Permissive auth provider for testing"""

        async def authenticate(self, sio: socketio.AsyncServer, environ: dict, auth: dict | None, headers: list) -> bool:  # type: ignore[override]
            return True

    class LocalAsyncSMCPNamespace(SMCPNamespace):
        """中文: 异步命名空间，继承自正式实现，仅替换认证 / English: Async namespace with test auth"""

        def __init__(self) -> None:
            super().__init__(auth_provider=_PassAsyncAuth())

    sio = socketio.AsyncServer(
        async_mode="asgi",
        cors_allowed_origins="*",
        ping_timeout=5,  # 中文: 测试环境使用较短超时 / English: Use shorter timeout for testing
        ping_interval=3,  # 中文: 测试环境使用较短间隔 / English: Use shorter interval for testing
        logger=False,
        engineio_logger=False,
    )
    # 避免关闭时后台任务异常 / avoid background task issues on shutdown
    sio.eio.start_service_task = False
    ns = LocalAsyncSMCPNamespace()
    sio.register_namespace(ns)
    app = socketio.ASGIApp(sio, socketio_path="/socket.io")
    return sio, ns, app


@pytest.fixture
def async_integration_server_port() -> int:
    """
    中文: 查找可用端口用于异步集成服务器。
    English: Find an available TCP port for async integration server.
    """
    sock = socket.socket()
    sock.bind(("127.0.0.1", 0))
    port = sock.getsockname()[1]
    sock.close()
    return port


@pytest.fixture
async def async_integration_socketio_server(async_integration_server_port: int):
    """
    中文: 启动基于 SMCPNamespace 的异步集成测试服务器，返回命名空间。
    English: Start async integration test server based on SMCPNamespace and return the namespace.
    """
    from tests.integration_tests.computer.socketio.mock_uv_server import UvicornTestServer

    setup_start = time.time()
    sio, ns, asgi_app = create_local_async_server()
    print(f"[E2E Fixture] Server creation took {time.time() - setup_start:.2f}s")

    server_start = time.time()
    server = UvicornTestServer(asgi_app, port=async_integration_server_port)
    await server.up()
    print(f"[E2E Fixture] Server startup took {time.time() - server_start:.2f}s")

    try:
        yield ns
    finally:
        shutdown_start = time.time()
        print(f"[E2E Fixture] Starting shutdown Server {shutdown_start}")
        # 强制快速关闭，不等待连接清理 / Force fast shutdown without waiting for connection cleanup
        await server.down(force=True)
        print(f"[E2E Fixture] Server shutdown took {time.time() - shutdown_start:.2f}s")



"""
文件名: cli_io.py
作者: JQQ
创建日期: 2025/9/18
最后修改日期: 2025/9/18
版权: 2023 JQQ. All rights reserved.
依赖: prompt_toolkit, rich
描述:
  中文: 命令行交互 I/O 抽象，提供异步的提示输入、密码输入、选项选择与命令执行封装。
  English: CLI interactive I/O abstraction. Provides async prompt input, password input,
           pick options and command execution wrappers.
"""

from __future__ import annotations

import asyncio
from typing import Any

from prompt_toolkit import PromptSession
from prompt_toolkit.patch_stdout import patch_stdout
from rich.table import Table

from a2c_smcp.computer.utils import console as console_util


async def ainput_prompt(
    message: str,
    *,
    password: bool = False,
    default: str | None = None,
    session: PromptSession | None = None,
) -> str:
    """
    中文: 提示用户输入字符串；当 password=True 时进行掩码输入。
    English: Prompt user for a string input; when password=True, mask the input.
    """
    sess = session or PromptSession()
    prompt_text = f"{message} " + (f"[default: {repr(default)}] " if default is not None else "")
    # 中文: 若未提供 session，说明在交互循环外部调用，为避免与 a2c> 冲突，强制换行并启用 raw。
    # English: If no session provided (outside interactive loop), start on a new line and enable raw to avoid conflicts.
    value: str
    if session is None:
        prompt_text = "\n" + prompt_text
        with patch_stdout(raw=True):
            try:
                value = str(await sess.prompt_async(prompt_text, is_password=password))
            except (EOFError, KeyboardInterrupt):
                return default or ""
    else:
        # 中文: 复用交互循环的 session，不额外换行；由外层 prompt_toolkit 负责正确重绘与光标管理。
        # English: Reuse interactive loop session; no extra newline; rely on outer prompt_toolkit for redraw/cursor.
        try:
            value = str(await sess.prompt_async(prompt_text, is_password=password))
        except (EOFError, KeyboardInterrupt):
            return default or ""
    if value == "" and default is not None:
        return default
    return value


async def ainput_pick(
    message: str,
    options: list[str] | tuple[str, ...],
    *,
    default_index: int | None = None,
    multi: bool = False,
    session: PromptSession | None = None,
) -> None | list[str] | str | list[Any]:
    """
    中文: 让用户以序号选择一个或多个字符串。
    English: Let user pick one or multiple strings by index.
    """
    if not options:
        return [] if multi else ""

    table = Table(title=message)
    table.add_column("#", justify="right")
    table.add_column("Option", overflow="fold")
    for idx, opt in enumerate(options):
        table.add_row(str(idx), opt)
    console_util.console.print(table)

    tip = "输入序号，多个用逗号分隔" if multi else "输入序号"
    if default_index is not None and 0 <= default_index < len(options):
        tip += f"（默认: {default_index}）"

    sess = session or PromptSession()
    while True:
        try:
            raw = await sess.prompt_async(f"{tip}: ")
        except (EOFError, KeyboardInterrupt):
            if default_index is not None and 0 <= default_index < len(options):
                return [options[default_index]] if multi else options[default_index]
            return [] if multi else ""

        raw = raw.strip()
        if raw == "" and default_index is not None and 0 <= default_index < len(options):
            return [options[default_index]] if multi else options[default_index]

        try:
            if multi:
                idxs = [int(x.strip()) for x in raw.split(",") if x.strip() != ""]
                if any(i < 0 or i >= len(options) for i in idxs):
                    console_util.console.print("[yellow]序号越界，请重试 / Index out of range, please retry[/yellow]")
                    continue
                picked = [options[i] for i in idxs]
                # 去重保持顺序
                seen: set[str] = set()
                result: list[str] = []
                for p in picked:
                    if p not in seen:
                        seen.add(p)
                        result.append(p)
                return result
            else:
                idx = int(raw)
                if idx < 0 or idx >= len(options):
                    console_util.console.print("[yellow]序号越界，请重试 / Index out of range, please retry[/yellow]")
                    continue
                return options[idx]
        except ValueError:
            console_util.console.print("[yellow]请输入有效的数字 / Please enter valid number(s)[/yellow]")


async def arun_command(
    command: str,
    *,
    cwd: str | None = None,
    timeout: float | None = None,
    shell: bool = True,
    parse: str = "raw",
) -> Any:
    """
    中文: 异步执行命令并返回结果，支持超时、工作目录与解析模式。
    English: Run a command asynchronously with timeout, cwd, and parse modes.
    parse: 'raw' | 'lines' | 'json'
    """
    if shell:
        proc = await asyncio.create_subprocess_shell(
            command,
            cwd=cwd,
            stdout=asyncio.subprocess.PIPE,
            stderr=asyncio.subprocess.PIPE,
        )
    else:
        # 简化处理：当 shell=False 时仅将 command 作为可执行名，不拆分参数
        proc = await asyncio.create_subprocess_exec(
            command,
            cwd=cwd,
            stdout=asyncio.subprocess.PIPE,
            stderr=asyncio.subprocess.PIPE,
        )

    try:
        if timeout:
            stdout, stderr = await asyncio.wait_for(proc.communicate(), timeout)
        else:
            stdout, stderr = await proc.communicate()
    except TimeoutError:
        try:
            proc.kill()
        except Exception:
            pass
        raise TimeoutError(f"Command timeout: {command}") from None

    rc = proc.returncode
    out = stdout.decode(errors="ignore") if stdout else ""
    err = stderr.decode(errors="ignore") if stderr else ""
    if rc != 0:
        # 保留原始输出，交由上层决定是否继续
        raise RuntimeError(f"Command failed (rc={rc}): {command}\nSTDERR: {err.strip()}")

    out = out.strip()
    if parse == "lines":
        return [line for line in out.splitlines() if line.strip() != ""]
    if parse == "json":
        import json

        try:
            return json.loads(out)
        except Exception:
            # 回退为原始文本
            return out
    return out



# -*- coding: utf-8 -*-
# filename: _local_sync_server.py
# @Time    : 2025/09/30 23:28
# @Author  : A2C-SMCP
"""
中文：仅供本测试包使用的同步 SMCP 服务端（基于 SyncSMCPNamespace），使用放行认证。
English: Sync SMCP server for this test package only (based on SyncSMCPNamespace) with permissive auth.
"""

from typing import Any

from socketio import Namespace, Server, WSGIApp

from a2c_smcp.server import SyncSMCPNamespace
from a2c_smcp.server.sync_auth import SyncAuthenticationProvider


class _PassSyncAuth(SyncAuthenticationProvider):
    def authenticate(self, sio: Server, environ: dict, auth: dict | None, headers: list) -> bool:  # type: ignore[override]
        return True


class LocalSyncSMCPNamespace(SyncSMCPNamespace):
    """同步命名空间，继承自正式实现，仅替换认证。"""

    def __init__(self) -> None:
        super().__init__(auth_provider=_PassSyncAuth())
        # 可选：记录操作
        self.client_operations_record: dict[str, tuple[str, Any]] = {}

    def on_connect(self, sid: str, environ: dict, auth: dict | None = None) -> bool:  # type: ignore[override]
        ok = super().on_connect(sid, environ, auth)
        self.client_operations_record[sid] = ("connect", None)
        return ok


def create_local_sync_server() -> tuple[Server, Namespace, WSGIApp]:
    """创建同步 Socket.IO Server 并注册本地命名空间，返回 (sio, namespace, wsgi_app)。"""
    sio = Server(
        cors_allowed_origins="*",
        ping_timeout=60,
        ping_interval=25,
        async_handlers=True,  # 如果想使用 call 方法，则必定需要将此参数设置为True
        always_connect=True,
    )
    ns = LocalSyncSMCPNamespace()
    sio.register_namespace(ns)
    app = WSGIApp(sio, socketio_path="/socket.io")
    return sio, ns, app



# -*- coding: utf-8 -*-
# filename: test_utils.py
# @Time    : 2025/8/19 16:58
# @Author  : JQQ
# @Email   : jqq1716@gmail.com
# @Software: PyCharm
"""
测试 client_factory 工厂方法，确保不同类型配置能正确实例化对应客户端。
Test client_factory factory method to ensure correct client instantiation for each config type.
"""

import pytest
from polyfactory.factories.pydantic_factory import ModelFactory

from a2c_smcp.computer.mcp_clients.http_client import HttpMCPClient
from a2c_smcp.computer.mcp_clients.model import SseServerConfig, StdioServerConfig, StreamableHttpServerConfig
from a2c_smcp.computer.mcp_clients.sse_client import SseMCPClient
from a2c_smcp.computer.mcp_clients.stdio_client import StdioMCPClient
from a2c_smcp.computer.mcp_clients.utils import client_factory


class StdioServerConfigFactory(ModelFactory[StdioServerConfig]):
    __model__ = StdioServerConfig

    @classmethod
    def vrl(cls) -> str | None:
        """生成有效的VRL脚本或None / Generate valid VRL script or None"""
        return cls.__random__.choice([None, '.result = "success"', ".transformed = true"])


class SseServerConfigFactory(ModelFactory[SseServerConfig]):
    __model__ = SseServerConfig

    @classmethod
    def vrl(cls) -> str | None:
        """生成有效的VRL脚本或None / Generate valid VRL script or None"""
        return cls.__random__.choice([None, '.result = "success"', ".transformed = true"])


class StreamableHttpServerConfigFactory(ModelFactory[StreamableHttpServerConfig]):
    __model__ = StreamableHttpServerConfig

    @classmethod
    def vrl(cls) -> str | None:
        """生成有效的VRL脚本或None / Generate valid VRL script or None"""
        return cls.__random__.choice([None, '.result = "success"', ".transformed = true"])


@pytest.mark.parametrize(
    "config_factory,expected_type",
    [
        (StdioServerConfigFactory, StdioMCPClient),
        (SseServerConfigFactory, SseMCPClient),
        (StreamableHttpServerConfigFactory, HttpMCPClient),
    ],
)
def test_client_factory_type(config_factory, expected_type):
    """
    检查 client_factory 根据不同配置类型返回正确的客户端类型。
    Check client_factory returns the correct client type for each config type.
    """
    config = config_factory.build()
    client = client_factory(config)
    assert isinstance(client, expected_type)


def test_client_factory_invalid_type():
    """
    检查 client_factory 对不支持的类型抛出异常。
    Check client_factory raises exception for unsupported config types.
    """

    class DummyConfig:
        pass

    with pytest.raises(ValueError):
        client_factory(DummyConfig())  # noqa



"""
中文: 发送 listChanged 通知的最小 MCP Stdio 服务器，用于集成测试。
英文: Minimal MCP Stdio server that sends listChanged notifications for integration tests.

文件名: notifications_stdio_server.py
作者: JQQ
创建日期: 2025/9/28
最后修改日期: 2025/9/28
版权: 2023 JQQ. All rights reserved.
依赖: anyio, mcp
描述: 提供触发工具/资源/提示列表变更通知的能力，并包含基础 list_* 与 call_tool 实现。
"""

import anyio
import mcp.types as types
from mcp.server.lowlevel.server import NotificationOptions, Server
from mcp.server.stdio import stdio_server


async def run() -> None:
    """
    中文: 启动服务器，注册基础 list_* 与一个触发变更通知的工具。
    英文: Start server, register basic list_* and a tool that triggers change notifications.
    """
    server = Server(name="notify-itest-server", version="0.0.1", instructions="itest-notify")

    @server.list_tools()
    async def handle_list_tools() -> list[types.Tool]:
        return [
            types.Tool(
                name="trigger_changes",
                description="Trigger tools/resources/prompts listChanged notifications",
                inputSchema={"type": "object", "properties": {}},
            ),
        ]

    @server.list_resources()
    async def handle_list_resources() -> list[types.Resource]:
        return []

    @server.list_prompts()
    async def handle_list_prompts() -> list[types.Prompt]:
        return []

    @server.call_tool()
    async def handle_call_tool(name: str, arguments: dict | None):
        ctx = server.request_context
        if name == "trigger_changes":
            # 依次发送列表变更通知 / send list-changed notifications
            await ctx.session.send_tool_list_changed()
            await ctx.session.send_resource_list_changed()
            await ctx.session.send_prompt_list_changed()
            return [types.TextContent(type="text", text="changes triggered")]
        return [types.TextContent(type="text", text=f"unknown tool: {name}")]

    async with stdio_server() as (read_stream, write_stream):
        init_opts = server.create_initialization_options(
            notification_options=NotificationOptions(prompts_changed=True, resources_changed=True, tools_changed=True),
        )
        await server.run(read_stream, write_stream, init_opts)


if __name__ == "__main__":
    anyio.run(run)



"""
文件名: test_cli_tc.py
作者: JQQ
创建日期: 2025/10/02
最后修改日期: 2025/10/02
版权: 2023 JQQ. All rights reserved.
依赖: pytest, pexpect
描述:
  中文:
    - 端到端验证交互式 CLI 的 `tc` 命令，针对真实 stdio MCP server（direct_execution.py）。
    - 通过写入 server 配置文件并使用 `server add @file` + `start <name>` 启动。
    - 为避免二次确认阻断，使用 `tool_meta.hello.auto_apply=true` 开启自动执行。
    - 发送 `tc` 的 JSON 负载（与 Socket.IO 一致的 `ToolCallReq` 结构），期望输出包含工具返回文本。

  English:
    - E2E test for interactive CLI `tc` against real stdio MCP server (direct_execution.py).
    - Start server via config file and `server add @file` + `start <name>`.
    - Enable `tool_meta.hello.auto_apply=true` to bypass confirm gate.
    - Send `tc` JSON payload (Socket.IO-compatible `ToolCallReq`) and expect tool output text.

测试要点与断言:
  1) 成功添加并启动名为 `e2e-tc` 的 stdio 服务器
  2) `tools` 输出包含 `hello`
  3) 发送 `tc`（tool_name 使用 `e2e-tc/hello` 以确保路由）后输出包含 `Hello, E2E!`
"""

from __future__ import annotations

import json
import sys
from pathlib import Path

import pytest

from tests.e2e.computer.utils import expect_prompt_stable, strip_ansi

pexpect = pytest.importorskip("pexpect", reason="e2e tests require pexpect; install with `pip install pexpect`.")


@pytest.mark.e2e
def test_tc_call_hello(cli_proc: pexpect.spawn, tmp_path: Path) -> None:
    """
    中文: 通过 `tc` 调用 `hello` 工具，并校验结果文本。
    English: Call `hello` tool via `tc` and assert result text.
    """
    child = cli_proc

    # 1) 写入 server 配置文件（打开 auto_apply），指向 direct_execution.py
    server_cfg = {
        "name": "e2e-tc",
        "type": "stdio",
        "disabled": False,
        "forbidden_tools": [],
        "tool_meta": {"hello": {"auto_apply": True}},
        "server_parameters": {
            "command": sys.executable,  # 使用当前 Python 解释器 / Use current Python interpreter
            "args": [
                "tests/integration_tests/computer/mcp_servers/direct_execution.py",
            ],
            "env": None,
            "cwd": None,
            "encoding": "utf-8",
            "encoding_error_handler": "strict",
        },
    }
    cfg_path = tmp_path / "server_e2e_tc.json"
    cfg_path.write_text(json.dumps(server_cfg, ensure_ascii=False), encoding="utf-8")

    # 2) 添加配置并启动
    child.sendline(f"server add @{cfg_path}")
    expect_prompt_stable(child, quiet=0.6, max_wait=15.0)
    child.sendline("start e2e-tc")
    expect_prompt_stable(child, quiet=0.6, max_wait=15.0)

    # 3) 确认工具已可见（tools 列表应包含 hello）
    child.sendline("tools")
    tools_out = expect_prompt_stable(child, quiet=0.6, max_wait=12.0)
    assert "hello" in strip_ansi(tools_out)

    # 4) 构造 tc 负载，工具名使用原始 MCP 工具名（不加前缀）。
    #    中文: Manager 会在所有已启动 server 中解析该工具名；我们前一步已确认 tools 中包含 hello。
    #    English: Manager resolves plain tool name across active servers; tools list already contains 'hello'.
    tc_payload = {
        "agent": "bot-e2e",
        "req_id": "req-e2e-hello",
        "computer": "ignored",
        "tool_name": "hello",
        "params": {"name": "E2E"},
        "timeout": 10,
    }

    child.sendline(f"tc {json.dumps(tc_payload, ensure_ascii=False)}")
    out = expect_prompt_stable(child, quiet=0.8, max_wait=20.0)
    out = strip_ansi(out)

    # 5) 断言包含调用结果文本
    assert "Hello, E2E!" in out, f"unexpected tc output:\n{out}"


@pytest.mark.e2e
def test_tc_default_and_tool_both_false_then_error(cli_proc: pexpect.spawn, tmp_path: Path) -> None:
    """
    中文: 当 `tool_meta.mark_b.auto_apply=False` 且 `default_tool_meta.auto_apply=False` 时，调用需要二次确认，
        但 CLI 未实现回调，应返回错误提示文本。
    English: With both `tool_meta.mark_b.auto_apply=False` and `default_tool_meta.auto_apply=False`, call requires
        confirm; CLI lacks confirm callback, expect error message.
    """
    child = cli_proc

    server_cfg = {
        "name": "e2e-tc-b-both-false",
        "type": "stdio",
        "disabled": False,
        "forbidden_tools": [],
        "tool_meta": {"mark_b": {"auto_apply": False}},
        "default_tool_meta": {"auto_apply": False},
        "server_parameters": {
            "command": sys.executable,  # 使用当前 Python 解释器 / Use current Python interpreter
            "args": [
                "tests/integration_tests/computer/mcp_servers/resources_subscribe_b_stdio_server.py",
            ],
            "env": None,
            "cwd": None,
            "encoding": "utf-8",
            "encoding_error_handler": "strict",
        },
    }
    cfg_path = tmp_path / "server_e2e_tc_b_both_false.json"
    cfg_path.write_text(json.dumps(server_cfg, ensure_ascii=False), encoding="utf-8")

    child.sendline(f"server add @{cfg_path}")
    expect_prompt_stable(child, quiet=0.6, max_wait=15.0)
    child.sendline("start e2e-tc-b-both-false")
    expect_prompt_stable(child, quiet=0.6, max_wait=15.0)

    # 发送 tc（使用 mark_b）
    tc_payload = {
        "agent": "r-e2e",
        "req_id": "req-e2e",
        "computer": "client",
        "tool_name": "mark_b",
        "params": {},
        "timeout": 5,
    }
    child.sendline(f"tc {json.dumps(tc_payload, ensure_ascii=False)}")
    out = expect_prompt_stable(child, quiet=0.8, max_wait=20.0)
    out = strip_ansi(out)

    # 期望错误并给出中文提示
    assert '"isError": true' in out or '"isError":true' in out
    assert "当前工具需要调用前进行二次确认" in out, f"unexpected output:\n{out}"


@pytest.mark.e2e
def test_tc_tool_true_overrides_default(cli_proc: pexpect.spawn, tmp_path: Path) -> None:
    """
    中文: 如果工具设置了 auto_apply=True，则无论 default_tool_meta 如何，都应直接执行成功。
    English: If tool's auto_apply=True is set, it should override default and execute successfully.
    """
    child = cli_proc

    server_cfg = {
        "name": "e2e-tc-b-tool-true",
        "type": "stdio",
        "disabled": False,
        "forbidden_tools": [],
        # 工具设置为 True，default 故意设为 False 以验证覆盖
        "tool_meta": {"mark_b": {"auto_apply": True}},
        "default_tool_meta": {"auto_apply": False},
        "server_parameters": {
            "command": sys.executable,  # 使用当前 Python 解释器 / Use current Python interpreter
            "args": [
                "tests/integration_tests/computer/mcp_servers/resources_subscribe_b_stdio_server.py",
            ],
            "env": None,
            "cwd": None,
            "encoding": "utf-8",
            "encoding_error_handler": "strict",
        },
    }
    cfg_path = tmp_path / "server_e2e_tc_b_tool_true.json"
    cfg_path.write_text(json.dumps(server_cfg, ensure_ascii=False), encoding="utf-8")

    child.sendline(f"server add @{cfg_path}")
    expect_prompt_stable(child, quiet=0.6, max_wait=15.0)
    child.sendline("start e2e-tc-b-tool-true")
    expect_prompt_stable(child, quiet=0.6, max_wait=15.0)

    tc_payload = {
        "agent": "r-e2e",
        "req_id": "req-e2e",
        "computer": "client",
        "tool_name": "mark_b",
        "params": {},
        "timeout": 5,
    }
    child.sendline(f"tc {json.dumps(tc_payload, ensure_ascii=False)}")
    out = expect_prompt_stable(child, quiet=0.8, max_wait=20.0)
    out = strip_ansi(out)

    assert "ok:mark_b" in out and '"isError": false' in out or '"isError":false' in out


@pytest.mark.e2e
def test_tc_tool_unset_uses_default(cli_proc: pexpect.spawn, tmp_path: Path) -> None:
    """
    中文: 当工具未设置 auto_apply 时，遵循 default_tool_meta；这里 default 设置为 True，应执行成功。
    English: When tool auto_apply is unset, follow default_tool_meta; with default=True, expect success.
    """
    child = cli_proc

    server_cfg = {
        "name": "e2e-tc-b-default-true",
        "type": "stdio",
        "disabled": False,
        "forbidden_tools": [],
        # 不设置 tool_meta.mark_b
        "tool_meta": {},
        "default_tool_meta": {"auto_apply": True},
        "server_parameters": {
            "command": sys.executable,  # 使用当前 Python 解释器 / Use current Python interpreter
            "args": [
                "tests/integration_tests/computer/mcp_servers/resources_subscribe_b_stdio_server.py",
            ],
            "env": None,
            "cwd": None,
            "encoding": "utf-8",
            "encoding_error_handler": "strict",
        },
    }
    cfg_path = tmp_path / "server_e2e_tc_b_default_true.json"
    cfg_path.write_text(json.dumps(server_cfg, ensure_ascii=False), encoding="utf-8")

    child.sendline(f"server add @{cfg_path}")
    expect_prompt_stable(child, quiet=0.6, max_wait=15.0)
    child.sendline("start e2e-tc-b-default-true")
    expect_prompt_stable(child, quiet=0.6, max_wait=15.0)

    tc_payload = {
        "agent": "r-e2e",
        "req_id": "req-e2e",
        "computer": "client",
        "tool_name": "mark_b",
        "params": {},
        "timeout": 5,
    }
    child.sendline(f"tc {json.dumps(tc_payload, ensure_ascii=False)}")
    out = expect_prompt_stable(child, quiet=0.8, max_wait=20.0)
    out = strip_ansi(out)

    assert "ok:mark_b" in out and ('"isError": false' in out or '"isError":false' in out)



# -*- coding: utf-8 -*-
# filename: test_claude_stdio_tools.py
# 目的：连接本机的 Claude Code 以 stdio 方式运行的 MCP Server，列出工具与其入参 Schema
# 运行前提：已安装并可在 PATH 中找到 `claude` 可执行文件

from __future__ import annotations

import json
import shutil

import pytest
from mcp import StdioServerParameters

from a2c_smcp.computer.mcp_clients.stdio_client import StdioMCPClient


@pytest.mark.asyncio
async def test_claude_mcp_tools_list_and_params() -> None:
    """
    中文:
      - 若本机安装了 `claude`，以 stdio 方式启动 `claude mcp serve`
      - 连接后列出所有可用工具及其入参（inputSchema），并打印为 JSON 便于查看
    英文:
      - If `claude` CLI is available, start `claude mcp serve` via stdio
      - Connect and list available tools with their inputSchema, print as JSON
    """
    # 未安装则跳过，避免在 CI 或未配置环境下失败
    if not shutil.which("claude"):
        pytest.skip("`claude` CLI not found in PATH; skip Claude MCP integration test")

    # 如需提高输出上限，可在外部设置: export MAX_MCP_OUTPUT_TOKENS=50000
    params = StdioServerParameters(command="claude", args=["mcp", "serve"])  # env 可按需在此扩展

    client = StdioMCPClient(params)

    # 连接 / connect
    await client.aconnect()
    await client._create_session_success_event.wait()

    # 基础校验 / basic checks
    assert client.initialize_result is not None
    assert client.initialize_result.capabilities.tools, "Claude MCP server should expose tools capability"

    # 获取工具列表 / list tools
    tools = await client.list_tools()
    assert tools, "Claude MCP should expose at least one tool"

    collected: list[dict] = []
    for t in tools:
        # inputSchema 可能是 pydantic BaseModel 或原始 dict
        schema = getattr(t, "inputSchema", None)
        if hasattr(schema, "model_dump"):
            try:
                schema = schema.model_dump(mode="json")
            except Exception:
                # 兜底为可序列化对象
                schema = json.loads(json.dumps(schema, default=str))
        collected.append(
            {
                "name": t.name,
                "description": getattr(t, "description", None),
                "inputSchema": schema,
            },
        )

    # 打印用于人工查看（不作为断言依据） / print for manual inspection
    print(json.dumps(collected, ensure_ascii=False, indent=2))

    # 断开 / disconnect
    await client.adisconnect()
    await client._async_session_closed_event.wait()



# -*- coding: utf-8 -*-
# filename: __init__.py.py
# @Time    : 2025/8/15 11:47
# @Author  : JQQ
# @Email   : jqq1716@gmail.com
# @Software: PyCharm
from a2c_smcp.computer.computer import Computer
from a2c_smcp.computer.socketio.client import SMCPComputerClient

__all__ = ["Computer", "SMCPComputerClient"]



# filename: test_logger.py
# @Time    : 2025/8/15 19:36
# @Author  : JQQ
# @Email   : jqq1716@gmail.com
# @Software: PyCharm
import importlib
import logging
import sys
from io import StringIO

import pytest

# 待测试模块名
MODULE_NAME = "a2c_smcp.utils.logger"


@pytest.fixture(autouse=True)
def reset_logging():
    """每个测试后重置 logging 状态"""
    yield
    # 重置 logging 模块
    logging.root.handlers = []
    logging.root.setLevel(logging.NOTSET)
    logging.Logger.manager.loggerDict.clear()


def reload_module(monkeypatch, env_vars=None):
    """重新加载指定模块，设置环境变量"""
    if env_vars:
        for key, value in env_vars.items():
            monkeypatch.setenv(key, value)

    # 移除模块缓存以便重新加载
    if MODULE_NAME in sys.modules:
        del sys.modules[MODULE_NAME]

    # 捕获标准输出以验证日志
    captured_output = StringIO()
    monkeypatch.setattr(sys, "stdout", captured_output)

    # 重新加载模块
    module = importlib.import_module(MODULE_NAME)
    return module, captured_output


def test_default_config(monkeypatch):
    """测试默认配置"""
    module, captured_output = reload_module(monkeypatch, env_vars={"A2C_SMCP_LOG_LEVEL": "debug"})

    # 验证日志级别为 DEBUG
    assert module.logger.level == logging.DEBUG

    # 验证日志输出
    module.logger.info("Test info message")
    module.logger.debug("Test debug message (should not appear)")

    output = captured_output.getvalue()
    assert "Test info message" in output
    assert "Test debug message (should not appear)" in output
    assert "日志系统已初始化" in output


def test_silent_mode(monkeypatch):
    """测试静默模式"""
    module, captured_output = reload_module(monkeypatch, {"A2C_SMCP_LOG_SILENT": "true"})

    # 验证日志被禁用
    assert module.logger.disabled

    # 验证日志输出被禁用
    module.logger.info("This should not appear")
    module.logger.error("This should not appear either")

    output = captured_output.getvalue()
    assert "This should not appear" not in output
    assert output == ""  # 确保静默模式完全没有输出


def test_log_levels(monkeypatch):
    """测试所有日志级别"""
    test_cases = [
        ("debug", logging.DEBUG),
        ("info", logging.INFO),
        ("warning", logging.WARNING),
        ("error", logging.ERROR),
        ("critical", logging.CRITICAL),
        ("invalid", logging.INFO),  # 测试无效值默认为 INFO
    ]

    for level_str, expected_level in test_cases:
        module, captured_output = reload_module(monkeypatch, {"A2C_SMCP_LOG_LEVEL": level_str})
        logger = module.logger

        # 清除输出缓冲区
        captured_output.truncate(0)
        captured_output.seek(0)

        # 发送不同级别的测试日志
        logger.debug(f"{level_str} debug test")
        logger.info(f"{level_str} info test")
        logger.warning(f"{level_str} warning test")
        logger.error(f"{level_str} error test")
        logger.critical(f"{level_str} critical test")

        output = captured_output.getvalue()

        # 验证低于当前级别的日志不会被输出
        if expected_level > logging.DEBUG:
            assert f"{level_str} debug test" not in output
        if expected_level > logging.INFO:
            assert f"{level_str} info test" not in output
        if expected_level > logging.WARNING:
            assert f"{level_str} warning test" not in output
        if expected_level > logging.ERROR:
            assert f"{level_str} error test" not in output

        # 验证等于或高于当前级别的日志会被输出
        if expected_level <= logging.DEBUG:
            assert f"{level_str} debug test" in output
        if expected_level <= logging.INFO:
            assert f"{level_str} info test" in output
        if expected_level <= logging.WARNING:
            assert f"{level_str} warning test" in output
        if expected_level <= logging.ERROR:
            assert f"{level_str} error test" in output
        if expected_level <= logging.CRITICAL:
            assert f"{level_str} critical test" in output


def test_log_output_to_console(monkeypatch):
    """测试控制台日志输出"""
    module, captured_output = reload_module(monkeypatch)

    # 验证日志消息格式
    module.logger.warning("Test warning message")
    output = captured_output.getvalue()
    assert "WARNING" in output
    assert "Test warning message" in output
    assert "- a2c_smcp - WARNING - Test warning message" in output


def test_log_output_to_file(monkeypatch, tmp_path):
    """测试文件日志输出"""
    log_file = tmp_path / "test.log"

    module, captured_output = reload_module(monkeypatch, {"A2C_SMCP_LOG_FILE": str(log_file), "A2C_SMCP_LOG_LEVEL": "debug"})

    # 验证日志文件创建
    assert log_file.exists()

    # 写入日志
    test_message = "File output test message"
    module.logger.info(test_message)
    module.logger.debug("Debug message")

    # 验证文件内容
    file_content = log_file.read_text()
    assert test_message in file_content
    assert "Debug message" in file_content

    # 验证控制台仍有输出
    output = captured_output.getvalue()
    assert test_message in output


def test_log_output_to_file_directory_creation(monkeypatch, tmp_path):
    """测试日志文件目录自动创建"""
    log_file = tmp_path / "non_existent_dir" / "test.log"
    assert not log_file.parent.exists()  # 确保目录不存在

    module, _ = reload_module(monkeypatch, {"A2C_SMCP_LOG_FILE": str(log_file)})

    # 验证目录被创建
    assert log_file.parent.exists()

    # 写入日志
    test_message = "Directory creation test"
    module.logger.info(test_message)

    # 验证日志文件创建成功
    assert log_file.exists()
    assert test_message in log_file.read_text()


def test_logger_independence(monkeypatch):
    """验证我们配置的logger不影响其他logger"""
    # 配置我们的logger为静默
    module, _ = reload_module(monkeypatch, {"A2C_SMCP_LOG_SILENT": "1"})

    # 创建另一个独立的logger
    other_logger = logging.getLogger("other_logger")
    other_logger.setLevel(logging.INFO)

    # 捕获其他logger的输出
    captured_output = StringIO()
    handler = logging.StreamHandler(captured_output)
    other_logger.addHandler(handler)

    # 记录消息
    other_logger.info("Independent logger test")

    # 验证我们的logger处于静默状态
    module.logger.info("This should be silent")
    assert captured_output.getvalue() == "Independent logger test\n"

    # 验证我们logger的静默设置不影响其他logger
    assert "Independent logger test" in captured_output.getvalue()
    assert "This should be silent" not in captured_output.getvalue()


def test_log_format_customization(monkeypatch):
    """测试日志格式（虽然我们不能通过env改变格式，但检查默认格式）"""
    module, captured_output = reload_module(monkeypatch)

    module.logger.info("Format test")
    output = captured_output.getvalue()

    # 验证日志格式组件
    assert " - a2c_smcp - " in output
    assert " - Format test" in output

    # 验证时间戳格式
    from datetime import datetime

    timestamp = output.split(" - ")[0]
    print(datetime.strptime(timestamp, "%Y-%m-%d %H:%M:%S,%f"))


def test_multiple_handlers(monkeypatch, tmp_path):
    """测试同时输出到控制台和文件"""
    log_file = tmp_path / "combined.log"

    module, captured_output = reload_module(monkeypatch, {"A2C_SMCP_LOG_FILE": str(log_file), "A2C_SMCP_LOG_LEVEL": "debug"})

    # 验证有两个handler
    assert len(module.logger.handlers) == 2

    # 写入日志
    test_message = "Multiple handlers test"
    module.logger.info(test_message)

    # 验证控制台输出
    output = captured_output.getvalue()
    assert test_message in output

    # 验证文件输出
    file_content = log_file.read_text()
    assert test_message in file_content


def test_silent_mode_with_file(monkeypatch, tmp_path):
    """测试静默模式下不输出到文件"""
    log_file = tmp_path / "silent.log"

    module, captured_output = reload_module(monkeypatch, {"A2C_SMCP_LOG_SILENT": "yes", "A2C_SMCP_LOG_FILE": str(log_file)})

    # 验证日志被禁用
    assert module.logger.disabled

    # 尝试写入日志
    module.logger.info("Should not appear")

    # 验证控制台无输出
    assert captured_output.getvalue() == ""

    # 验证无日志文件创建
    assert not log_file.exists()



# -*- coding: utf-8 -*-
# filename: test_manager.py
# @Time    : 2025/11/21 12:19
# @Author  : JQQ
# @Email   : jqq1716@gmail.com
# @Software: PyCharm

"""
MCP Manager E2E测试 / MCP Manager E2E Tests

测试真实MCP服务器与VRL转换的集成功能。
Tests integration with real MCP servers and VRL transformation.
"""

import json

import pytest
from mcp import StdioServerParameters

from a2c_smcp.computer.mcp_clients.manager import MCPServerManager
from a2c_smcp.computer.mcp_clients.model import A2C_VRL_TRANSFORMED, StdioServerConfig, ToolMeta


@pytest.mark.asyncio
async def test_playwright_browser_navigate_with_vrl():
    """
    中文: 测试使用Playwright MCP服务的browser_navigate工具打开百度首页，并通过VRL重构返回结果
    English: Test using Playwright MCP's browser_navigate tool to open Baidu homepage with VRL transformation
    """
    # 中文: 定义VRL脚本，将返回结果重构为指定的JSON结构
    # English: Define VRL script to restructure the result into specified JSON format
    vrl_script = """
# 中文: 只对 browser_navigate 和 browser_navigate_back 工具进行转换
# English: Only transform for browser_navigate and browser_navigate_back tools
if .tool_name == "browser_navigate" || .tool_name == "browser_navigate_back" {
    # 中文: 提取URL，browser_navigate_back 可能没有url参数
    # English: Extract URL, browser_navigate_back may not have url parameter
    url = if exists(.parameters.url) {
        .parameters.url
    } else {
        "[BROWSER_BACK_OPERATION]"
    }

    # 中文: 提取内容 / English: Extract content
    content = if length!(.content) > 0 {
        .content[0].text
    } else {
        ""
    }

    # 中文: 使用重新赋值方式，只保留需要的字段
    # English: Use reassignment to keep only needed fields
    . = {
        "url": url,
        "content": content
    }
}
"""

    # 中文: 创建Playwright MCP服务配置
    # English: Create Playwright MCP server configuration
    playwright_config = StdioServerConfig(
        name="playwright",
        type="stdio",
        disabled=False,
        forbidden_tools=[],
        tool_meta={},
        default_tool_meta=ToolMeta(**{
            "tags": ["browser"],
            "auto_apply": True,
        }),
        vrl=vrl_script,
        server_parameters=StdioServerParameters(
            command="npx",
            args=["@playwright/mcp@latest"],
            env=None,
        ),
    )

    # 中文: 初始化Manager并添加服务器配置
    # English: Initialize Manager and add server configuration
    manager = MCPServerManager(auto_connect=True)
    await manager.aadd_or_aupdate_server(playwright_config)

    try:
        # 中文: 调用browser_navigate工具打开百度首页
        # English: Call browser_navigate tool to open Baidu homepage
        baidu_url = "https://www.baidu.com"
        result = await manager.acall_tool(
            server_name="playwright",
            tool_name="browser_navigate",
            parameters={"url": baidu_url},
            timeout=30.0,  # 中文: 浏览器操作可能需要较长时间 / English: Browser operations may take longer
        )

        # 中文: 验证返回结果的meta中包含VRL转换结果
        # English: Verify that meta contains VRL transformation result
        assert result.meta is not None, "Result meta should not be None"
        assert A2C_VRL_TRANSFORMED in result.meta, f"Result meta should contain {A2C_VRL_TRANSFORMED}"

        # 中文: 解析VRL转换后的JSON结构
        # English: Parse VRL transformed JSON structure
        transformed_json = json.loads(result.meta[A2C_VRL_TRANSFORMED])

        # 中文: 验证转换后的结构符合要求
        # English: Verify transformed structure meets requirements
        assert "url" in transformed_json, "Transformed result should contain 'url' field"
        assert "content" in transformed_json, "Transformed result should contain 'content' field"

        # 中文: 验证URL字段值正确
        # English: Verify URL field value is correct
        assert transformed_json["url"] == baidu_url, f"URL should be {baidu_url}"

        # 中文: 验证content字段存在（内容可能为空或包含页面信息）
        # English: Verify content field exists (content may be empty or contain page info)
        assert isinstance(transformed_json["content"], str), "Content should be a string"

        # 中文: 验证只包含url和content两个字段（VRL脚本删除了其他字段）
        # English: Verify only url and content fields exist (VRL script deleted other fields)
        assert set(transformed_json.keys()) == {
            "url",
            "content",
        }, "Transformed result should only contain 'url' and 'content' fields"

        # 中文: 打印转换结果用于调试
        # English: Print transformed result for debugging
        print(f"\n转换后的结果 / Transformed result:\n{json.dumps(transformed_json, indent=2, ensure_ascii=False)}")

    finally:
        # 中文: 清理：停止所有客户端
        # English: Cleanup: stop all clients
        await manager.astop_all()



# -*- coding: utf-8 -*-
# filename: test_param_server.py
# 集成测试：验证自定义参数化 stdio MCP Server 的动态配置更新能力
from __future__ import annotations

import sys
from pathlib import Path

import pytest
from mcp import StdioServerParameters

from a2c_smcp.computer.computer import Computer
from a2c_smcp.computer.mcp_clients.model import StdioServerConfig


@pytest.mark.asyncio
async def test_param_server_dynamic_update() -> None:
    """
    1) 启动带 GREETING 环境变量的 param_server（Hello）
    2) 调用工具 config_value -> 期望 "Hello, World!"
    3) 动态更新配置，将 GREETING 修改为 "Hi"，启用自动重连
    4) 再次调用 -> 期望 "Hi, World!"
    """
    server_script = Path(__file__).with_name("param_server.py")

    params = StdioServerParameters(
        command=sys.executable,
        args=[str(server_script)],
        env={"GREETING": "Hello"},
        cwd=None,
        encoding="utf-8",
        encoding_error_handler="strict",
    )
    cfg = StdioServerConfig(name="param_server", server_parameters=params)

    comp = Computer(name="test", inputs=set(), mcp_servers={cfg}, auto_connect=False, auto_reconnect=True)
    async with comp:
        # 启动该 server
        await comp.mcp_manager.astart_client("param_server")

        # 第一次调用
        result1 = await comp.mcp_manager.acall_tool("param_server", "config_value", {})
        assert not result1.isError
        assert result1.content and result1.content[0].type == "text"
        assert result1.content[0].text == "Hello, World!"

        # 更新配置 GREETING -> Hi
        new_params = StdioServerParameters(
            command=sys.executable,
            args=[str(server_script)],
            env={"GREETING": "Hi"},
            cwd=None,
            encoding="utf-8",
            encoding_error_handler="strict",
        )
        new_cfg = StdioServerConfig(name="param_server", server_parameters=new_params)
        await comp.aadd_or_aupdate_server(new_cfg)

        # 再次调用，验证已生效
        result2 = await comp.mcp_manager.acall_tool("param_server", "config_value", {})
        assert not result2.isError
        assert result2.content and result2.content[0].type == "text"
        assert result2.content[0].text == "Hi, World!"



# -*- coding: utf-8 -*-
# filename: test_inputs.py
# @Time    : 2025/11/24 16:49
# @Author  : JQQ
# @Email   : jqq1716@gmail.com
# @Software: PyCharm

"""
集成测试：Computer inputs 定义加载与值缓存行为
Integration test: Computer inputs definition loading and value cache behavior
"""

import pytest

from a2c_smcp.computer.computer import Computer
from a2c_smcp.computer.mcp_clients.model import MCPServerPromptStringInput


@pytest.mark.anyio
async def test_update_inputs_and_list_values() -> None:
    """
    中文: 测试 update_inputs 加载定义后，list_input_values 为空是正常行为。
    English: Test that after update_inputs loads definitions, list_input_values is empty (expected behavior).

    验证点 / Verification points:
    1. update_inputs 成功加载 inputs 定义
    2. inputs 定义包含 default 值
    3. list_input_values 返回空字典（因为没有实际解析）
    4. inputs 属性能正确返回所有定义
    """
    # 创建 Computer 实例 / Create Computer instance
    computer = Computer(name="test_computer")

    # 准备测试数据：与用户提供的示例一致 / Prepare test data: same as user's example
    test_inputs = {
        MCPServerPromptStringInput(
            id="CLAUDE_CWD",
            type="promptString",
            description="Claude MCP server working directory",
            default="/Users/huruize/PycharmProjects/TfrobotSceneTests",
            password=False,
        ),
        MCPServerPromptStringInput(
            id="FEISHU_APP_ID",
            type="promptString",
            description="Feishu MCP App ID",
            default="cli_a86666c7f478500e",
            password=False,
        ),
        MCPServerPromptStringInput(
            id="FEISHU_APP_SECRET",
            type="promptString",
            description="Feishu MCP App Secret",
            default="nxlKxbeKGcPGe1kNDa11EboJwUkYxFn6",
            password=True,
        ),
    }

    # 执行 update_inputs / Execute update_inputs
    computer.update_inputs(test_inputs)

    # 验证 1: inputs 定义已加载 / Verify 1: inputs definitions are loaded
    loaded_inputs = computer.inputs
    assert len(loaded_inputs) == 3, "应该加载 3 个 inputs 定义 / Should load 3 input definitions"

    # 验证 2: 检查每个 input 的定义是否正确 / Verify 2: check each input definition
    input_ids = {inp.id for inp in loaded_inputs}
    assert "CLAUDE_CWD" in input_ids
    assert "FEISHU_APP_ID" in input_ids
    assert "FEISHU_APP_SECRET" in input_ids

    # 验证 3: 检查 default 值存在 / Verify 3: check default values exist
    for inp in loaded_inputs:
        assert inp.default is not None, f"Input {inp.id} 应该有 default 值 / should have default value"

    # 验证 4: list_input_values 应该返回空字典 / Verify 4: list_input_values should return empty dict
    # 这是关键验证点：加载定义后，缓存应该为空
    # This is the key verification: after loading definitions, cache should be empty
    cached_values = computer.list_input_values()
    assert cached_values == {}, (
        "加载 inputs 定义后，缓存应该为空（default 不会自动填充）/ "
        "After loading input definitions, cache should be empty (defaults are not auto-populated)"
    )

    # 验证 5: 单独获取每个 input 的值也应该返回 None / Verify 5: getting individual values should return None
    assert computer.get_input_value("CLAUDE_CWD") is None
    assert computer.get_input_value("FEISHU_APP_ID") is None
    assert computer.get_input_value("FEISHU_APP_SECRET") is None


@pytest.mark.anyio
async def test_inputs_value_after_manual_set() -> None:
    """
    中文: 测试手动设置 input 值后，list_input_values 能正确返回。
    English: Test that after manually setting input values, list_input_values returns correctly.
    """
    computer = Computer(name="test_computer")

    # 加载定义 / Load definitions
    test_inputs = {
        MCPServerPromptStringInput(
            id="TEST_INPUT",
            type="promptString",
            description="Test input",
            default="default_value",
            password=False,
        ),
    }
    computer.update_inputs(test_inputs)

    # 初始缓存为空 / Initial cache is empty
    assert computer.list_input_values() == {}

    # 手动设置值 / Manually set value
    success = computer.set_input_value("TEST_INPUT", "manually_set_value")
    assert success is True, "设置值应该成功 / Setting value should succeed"

    # 现在缓存应该有值 / Now cache should have value
    cached_values = computer.list_input_values()
    assert cached_values == {"TEST_INPUT": "manually_set_value"}

    # 获取单个值 / Get individual value
    value = computer.get_input_value("TEST_INPUT")
    assert value == "manually_set_value"


@pytest.mark.anyio
async def test_inputs_value_after_resolve() -> None:
    """
    中文: 测试通过 aresolve_by_id 解析后，值会被缓存。
    English: Test that after resolving via aresolve_by_id, value is cached.

    注意：此测试需要模拟用户输入，暂时跳过实际解析测试。
    Note: This test requires mocking user input, skipping actual resolve test for now.
    """
    computer = Computer(name="test_computer")

    test_inputs = {
        MCPServerPromptStringInput(
            id="RESOLVE_TEST",
            type="promptString",
            description="Resolve test input",
            default="default_value",
            password=False,
        ),
    }
    computer.update_inputs(test_inputs)

    # 初始缓存为空 / Initial cache is empty
    assert computer.list_input_values() == {}

    # 注意：实际的 aresolve_by_id 需要用户交互，这里仅验证接口存在
    # Note: Actual aresolve_by_id requires user interaction, only verify interface exists
    assert hasattr(computer, "_input_resolver")
    assert hasattr(computer._input_resolver, "aresolve_by_id")


@pytest.mark.anyio
async def test_set_input_value_with_default() -> None:
    """
    中文: 测试通过 get_input 获取定义后，可以手动将 default 值设置到缓存。
    English: Test that after getting input definition, can manually set default value to cache.

    这模拟了 CLI 中 'inputs value set <id>' 不带值参数时的行为。
    This simulates the behavior of 'inputs value set <id>' without value parameter in CLI.
    """
    computer = Computer(name="test_computer")

    # 加载定义 / Load definitions
    test_inputs = {
        MCPServerPromptStringInput(
            id="WITH_DEFAULT",
            type="promptString",
            description="Input with default",
            default="my_default_value",
            password=False,
        ),
        MCPServerPromptStringInput(
            id="WITHOUT_DEFAULT",
            type="promptString",
            description="Input without default",
            default=None,
            password=False,
        ),
    }
    computer.update_inputs(test_inputs)

    # 初始缓存为空 / Initial cache is empty
    assert computer.list_input_values() == {}

    # 场景 1: 获取有 default 值的 input 定义，并手动设置 default 到缓存
    # Scenario 1: Get input with default, manually set default to cache
    input_def = computer.get_input("WITH_DEFAULT")
    assert input_def is not None
    assert input_def.default == "my_default_value"

    # 手动设置 default 值到缓存 / Manually set default to cache
    success = computer.set_input_value("WITH_DEFAULT", input_def.default)
    assert success is True

    # 验证缓存 / Verify cache
    cached_values = computer.list_input_values()
    assert cached_values == {"WITH_DEFAULT": "my_default_value"}

    # 场景 2: 没有 default 值的 input
    # Scenario 2: Input without default
    input_def_no_default = computer.get_input("WITHOUT_DEFAULT")
    assert input_def_no_default is not None
    assert input_def_no_default.default is None

    # 尝试设置 None 也是可以的 / Setting None is also allowed
    success = computer.set_input_value("WITHOUT_DEFAULT", None)
    assert success is True

    # 验证缓存 / Verify cache
    cached_values = computer.list_input_values()
    assert cached_values == {"WITH_DEFAULT": "my_default_value", "WITHOUT_DEFAULT": None}



# -*- coding: utf-8 -*-
"""
文件名: test_interactive_more.py
作者: JQQ
创建日期: 2025/9/29
最后修改日期: 2025/9/29
版权: 2023 JQQ. All rights reserved.
依赖: pytest
描述:
  中文: 覆盖 `interactive_impl.py` 剩余分支：tools、server @file、socket 已连接/未加入、inputs 变体与 inputs value JSON 文本两类。
  English: Cover remaining branches in `interactive_impl.py`: tools, server @file, socket already-connected/not-joined,
    inputs variants, and inputs value JSON vs text.
"""

from __future__ import annotations

import json
from contextlib import contextmanager
from pathlib import Path
from typing import Any

import pytest

import a2c_smcp.computer.cli.main as cli_main
from a2c_smcp.computer.cli.main import _interactive_loop
from a2c_smcp.computer.computer import Computer


class FakePromptSession:
    """中文: 将脚本化命令注入交互循环。English: Feed scripted inputs to interactive loop."""

    def __init__(self, commands: list[str]) -> None:
        self._commands = commands

    async def prompt_async(self, *_: str, **__: Any) -> str:  # noqa: D401
        if not self._commands:
            raise EOFError
        return self._commands.pop(0)


@contextmanager
def no_patch_stdout():
    """No-op context manager to replace patch_stdout() in tests."""
    yield


class _Client:
    """中文: 覆盖 socket 分支需要的最小客户端桩对象。English: Minimal client stub for socket branches."""

    def __init__(self, *args: Any, **kwargs: Any) -> None:  # noqa: D401
        self.connected = False
        self.office_id: str | None = None
        self.updated = 0
        _Client.last = self  # type: ignore[attr-defined]

    async def connect(self, url: str, auth: dict | None = None, headers: dict | None = None) -> None:  # noqa: D401
        self.connected = True
        self._url = url
        self._auth = auth
        self._headers = headers

    async def join_office(self, office_id: str, computer_name: str) -> None:  # noqa: D401
        assert self.connected
        self.office_id = office_id
        self._comp = computer_name

    async def leave_office(self, office_id: str) -> None:  # noqa: D401
        assert self.connected
        self.office_id = None

    async def emit_update_config(self) -> None:  # noqa: D401
        self.updated += 1


@pytest.mark.asyncio
async def test_tools_and_inputs_update_single_and_list(tmp_path: Path, monkeypatch: pytest.MonkeyPatch) -> None:
    """
    - tools: 覆盖 aget_available_tools 分支
    - inputs add @file (数组) 与 update (单条)
    - inputs get 不存在与存在
    - inputs value set: JSON 与 纯文本 两类
    """
    monkeypatch.setattr(cli_main, "SMCPComputerClient", _Client)

    # 通过文件提供 inputs 数组
    inputs_file = tmp_path / "inputs.json"
    inputs_file.write_text(
        json.dumps(
            [
                {"id": "A", "type": "promptString", "description": "d1", "default": "x"},
                {"id": "B", "type": "pickString", "description": "d2", "options": ["1", "2"], "default": "1"},
            ],
        ),
        encoding="utf-8",
    )

    # 服务器配置文件（用于 server add @file 覆盖）
    server_file = tmp_path / "server.json"
    server_file.write_text(
        json.dumps(
            {
                "name": "s1",
                "type": "stdio",
                "disabled": True,
                "forbidden_tools": [],
                "tool_meta": {},
                "server_parameters": {
                    "command": "echo",
                    "args": [],
                    "env": None,
                    "cwd": None,
                    "encoding": "utf-8",
                    "encoding_error_handler": "strict",
                },
            },
        ),
        encoding="utf-8",
    )

    # 提前连接一次以覆盖 "已连接" 分支
    pre_connect = [
        "socket connect http://localhost:9999",
        "tools",
        # inputs add @file (数组)
        f"inputs add @{inputs_file}",
        # get 不存在
        "inputs get NOT_EXIST",
        # update 单条（覆盖 update 的非数组路径）
        'inputs update {"id":"A","type":"promptString","description":"dx","default":"y"}',
        # value set JSON
        'inputs value set A {"k":1}',
        # value set 文本
        "inputs value set A ptext",
        # server add @file 与通知
        f"server add @{server_file}",
        # socket 已连接提示
        "socket connect http://localhost:9999",
        # leave 未加入
        "socket leave",
        "exit",
    ]

    # Monkeypatch 输入与 patch_stdout
    monkeypatch.setattr(cli_main, "PromptSession", lambda: FakePromptSession(pre_connect))
    monkeypatch.setattr(cli_main, "patch_stdout", lambda raw: no_patch_stdout())

    # 注入 tools 的桩实现
    comp = Computer(name="test_im_c", inputs=set(), mcp_servers=set(), auto_connect=False, auto_reconnect=False)

    async def _fake_tools() -> list[dict[str, Any]]:
        return [{"name": "t1", "description": "desc", "return_schema": {}}]

    monkeypatch.setattr(comp, "aget_available_tools", _fake_tools)

    await _interactive_loop(comp)

    # 断言 server add 时触发了配置更新
    last: _Client = _Client.last  # type: ignore[assignment]
    assert last.updated >= 1


@pytest.mark.asyncio
async def test_server_rm_and_start_stop_single_with_errors(monkeypatch: pytest.MonkeyPatch) -> None:
    """
    - 覆盖 server rm <name> 的 happy path
    - 覆盖 start/stop <name> 的异常路径（触发 except 分支）
    """
    monkeypatch.setattr(cli_main, "SMCPComputerClient", _Client)

    commands = [
        # 初始化 manager 后再进入交互
        "exit",
    ]
    monkeypatch.setattr(cli_main, "PromptSession", lambda: FakePromptSession(commands))
    monkeypatch.setattr(cli_main, "patch_stdout", lambda raw: no_patch_stdout())

    comp = Computer(name="test_im_c", inputs=set(), mcp_servers=set(), auto_connect=False, auto_reconnect=False)
    await comp.boot_up()

    # 准备第二段命令：rm + start/stop 单个 + 异常
    cmds2 = [
        "server rm not-exist",  # aremove_server 安静返回，不影响覆盖
        "start one",
        "stop one",
        "exit",
    ]
    monkeypatch.setattr(cli_main, "PromptSession", lambda: FakePromptSession(cmds2))

    # 注入 manager 的异常行为
    class BadMgr(type(comp.mcp_manager)):  # type: ignore[misc]
        async def astart_client(self, name: str) -> None:  # type: ignore[override]
            raise RuntimeError("boom-start")

        async def astop_client(self, name: str) -> None:  # type: ignore[override]
            raise RuntimeError("boom-stop")

    comp.mcp_manager.__class__ = BadMgr  # type: ignore[attr-defined]

    await _interactive_loop(comp)
